# type: ignore
import base64
import binascii
import copy
import html
import json
import mimetypes
import io
import os
import re
import shutil
import subprocess
import sys
import tempfile
import traceback
import uuid
from typing import Any, Dict, List, Optional, Union
from urllib.parse import parse_qs, quote, unquote, urlparse, urlunparse

import mammoth
import markdownify
import pandas as pd
import pdfminer
import pdfminer.high_level
from pdfminer.converter import PDFPageAggregator
from pdfminer.layout import LAParams, LTImage, LTFigure
from pdfminer.pdfdevice import PDFDevice
from pdfminer.pdfdocument import PDFDocument
from pdfminer.pdfparser import PDFParser
from pdfminer.pdfpage import PDFPage
from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
import pptx
from pdfminer.image import ImageWriter
import time

import numpy as np
from PIL import Image

# 新增导入
from autocoder.rag.loaders.filter_utils import FilterRuleManager
from autocoder.rag.loaders.image_loader import ImageLoader

# File-format detection
import puremagic
import requests
from bs4 import BeautifulSoup
from loguru import logger

# Optional Transcription support
try:
    import pydub
    import speech_recognition as sr

    IS_AUDIO_TRANSCRIPTION_CAPABLE = True
except ModuleNotFoundError:
    pass

# Optional YouTube transcription support
try:
    from youtube_transcript_api import YouTubeTranscriptApi

    IS_YOUTUBE_TRANSCRIPT_CAPABLE = True
except ModuleNotFoundError:
    pass


class _CustomMarkdownify(markdownify.MarkdownConverter):
    """
    A custom version of markdownify's MarkdownConverter. Changes include:

    - Altering the default heading style to use '#', '##', etc.
    - Removing javascript hyperlinks.
    - Truncating images with large data:uri sources.
    - Ensuring URIs are properly escaped, and do not conflict with Markdown syntax
    """

    def __init__(self, **options: Any):
        options["heading_style"] = options.get("heading_style", markdownify.ATX)
        # Explicitly cast options to the expected type if necessary
        super().__init__(**options)

    def convert_hn(self, n: int, el: Any, text: str, convert_as_inline: bool) -> str:
        """Same as usual, but be sure to start with a new line"""
        if not convert_as_inline:
            if not re.search(r"^\n", text):
                return "\n" + super().convert_hn(n, el, text, convert_as_inline)  # type: ignore

        return super().convert_hn(n, el, text, convert_as_inline)  # type: ignore

    def convert_a(self, el: Any, text: str, convert_as_inline: bool):
        """Same as usual converter, but removes Javascript links and escapes URIs."""
        prefix, suffix, text = markdownify.chomp(text)  # type: ignore
        if not text:
            return ""
        href = el.get("href")
        title = el.get("title")

        # Escape URIs and skip non-http or file schemes
        if href:
            try:
                parsed_url = urlparse(href)  # type: ignore
                # type: ignore
                if parsed_url.scheme and parsed_url.scheme.lower() not in [
                    "http",
                    "https",
                    "file",
                ]:
                    return "%s%s%s" % (prefix, text, suffix)
                href = urlunparse(
                    parsed_url._replace(path=quote(unquote(parsed_url.path)))
                )  # type: ignore
            except ValueError:  # It's not clear if this ever gets thrown
                return "%s%s%s" % (prefix, text, suffix)

        # For the replacement see #29: text nodes underscores are escaped
        if (
            self.options["autolinks"]
            and text.replace(r"\_", "_") == href
            and not title
            and not self.options["default_title"]
        ):
            # Shortcut syntax
            return "<%s>" % href
        if self.options["default_title"] and not title:
            title = href
        title_part = ' "%s"' % title.replace('"', r"\"") if title else ""
        return (
            "%s[%s](%s%s)%s" % (prefix, text, href, title_part, suffix)
            if href
            else text
        )

    def convert_img(self, el: Any, text: str, convert_as_inline: bool) -> str:
        """Same as usual converter, but removes data URIs"""

        alt = el.attrs.get("alt", None) or ""
        src = el.attrs.get("src", None) or ""
        title = el.attrs.get("title", None) or ""
        title_part = ' "%s"' % title.replace('"', r"\"") if title else ""
        if (
            convert_as_inline
            and el.parent.name not in self.options["keep_inline_images_in"]
        ):
            return alt

        # Remove dataURIs
        if src.startswith("data:"):
            src = src.split(",")[0] + "..."

        return "![%s](%s%s)" % (alt, src, title_part)

    def convert_soup(self, soup: Any) -> str:
        try:
            # 设置递归深度限制，避免复杂文档导致的递归错误
            import sys

            original_limit = sys.getrecursionlimit()
            try:
                # 增加递归深度限制
                sys.setrecursionlimit(10000)  # 设置更高的递归限制
                return super().convert_soup(soup)  # type: ignore
            finally:
                # 恢复原始递归深度限制
                sys.setrecursionlimit(original_limit)
        except RecursionError:
            # 处理递归错误，尝试简化处理
            logger.warning(
                "RecursionError in convert_soup, falling back to simplified conversion"
            )
            # 返回简化的文本内容
            return self._simplified_convert(soup)

    def _simplified_convert(self, soup: Any) -> str:
        """简化的转换方法，用于处理复杂文档时的回退方案"""
        # 提取纯文本内容
        text = soup.get_text(separator="\n", strip=True)
        # 基本清理
        text = re.sub(r"\s+", " ", text)
        text = re.sub(r"\n{3,}", "\n\n", text)
        return text


class DocumentConverterResult:
    """The result of converting a document to text."""

    def __init__(self, title: Union[str, None] = None, text_content: str = ""):
        self.title: Union[str, None] = title
        self.text_content: str = text_content


class DocumentConverter:
    """Abstract superclass of all DocumentConverters."""

    def convert(
        self, local_path: str, **kwargs: Any
    ) -> Union[None, DocumentConverterResult]:
        raise NotImplementedError()


class PlainTextConverter(DocumentConverter):
    """Anything with content type text/plain"""

    def convert(
        self, local_path: str, **kwargs: Any
    ) -> Union[None, DocumentConverterResult]:
        # Guess the content type from any file extension that might be around
        content_type, _ = mimetypes.guess_type(
            "__placeholder" + kwargs.get("file_extension", "")
        )

        # Only accept text files
        if content_type is None:
            return None
        elif "text/" not in content_type.lower():
            return None

        text_content = ""
        with open(local_path, "rt", encoding="utf-8") as fh:
            text_content = fh.read()
        return DocumentConverterResult(
            title=None,
            text_content=text_content,
        )


class HtmlConverter(DocumentConverter):
    """Anything with content type text/html"""

    def convert(
        self, local_path: str, **kwargs: Any
    ) -> Union[None, DocumentConverterResult]:
        # Bail if not html
        extension = kwargs.get("file_extension", "")
        if extension.lower() not in [".html", ".htm"]:
            return None

        result = None
        with open(local_path, "rt", encoding="utf-8") as fh:
            result = self._convert(fh.read())

        return result

    def _convert(self, html_content: str) -> Union[None, DocumentConverterResult]:
        """Helper function that converts and HTML string."""

        # Parse the string
        soup = BeautifulSoup(html_content, "html.parser")

        # Remove javascript and style blocks
        for script in soup(["script", "style"]):
            script.extract()

        try:
            # Print only the main content
            body_elm = soup.find("body")
            webpage_text = ""
            if body_elm:
                webpage_text = _CustomMarkdownify().convert_soup(body_elm)
            else:
                webpage_text = _CustomMarkdownify().convert_soup(soup)

            assert isinstance(webpage_text, str)

            return DocumentConverterResult(
                title=None if soup.title is None else soup.title.string,
                text_content=webpage_text,
            )
        except Exception as e:
            # 如果转换过程中出现任何错误，尝试使用简化的方法提取文本
            logger.warning(
                f"Error in HTML conversion: {str(e)}. Falling back to simplified text extraction."
            )
            try:
                # 简化的文本提取
                text = soup.get_text(separator="\n", strip=True)
                # 基本清理
                text = re.sub(r"\s+", " ", text)
                text = re.sub(r"\n{3,}", "\n\n", text)

                return DocumentConverterResult(
                    title=None if soup.title is None else soup.title.string,
                    text_content=text,
                )
            except Exception as inner_e:
                # 如果简化提取也失败，记录错误并返回空结果
                logger.error(
                    f"Failed to extract text with simplified method: {str(inner_e)}"
                )
                return DocumentConverterResult(
                    title=None,
                    text_content=f"[文档转换失败] 无法提取内容: {str(e)}",
                )


class WikipediaConverter(DocumentConverter):
    """Handle Wikipedia pages separately, focusing only on the main document content."""

    def convert(
        self, local_path: str, **kwargs: Any
    ) -> Union[None, DocumentConverterResult]:
        # Bail if not Wikipedia
        extension = kwargs.get("file_extension", "")
        if extension.lower() not in [".html", ".htm"]:
            return None
        url = kwargs.get("url", "")
        if not re.search(r"^https?:\/\/[a-zA-Z]{2,3}\.wikipedia.org\/", url):
            return None

        # Parse the file
        soup = None
        with open(local_path, "rt", encoding="utf-8") as fh:
            soup = BeautifulSoup(fh.read(), "html.parser")

        # Remove javascript and style blocks
        for script in soup(["script", "style"]):
            script.extract()

        # Print only the main content
        body_elm = soup.find("div", {"id": "mw-content-text"})
        title_elm = soup.find("span", {"class": "mw-page-title-main"})

        webpage_text = ""
        main_title = None if soup.title is None else soup.title.string

        if body_elm:
            # What's the title
            if title_elm and len(title_elm) > 0:
                main_title = title_elm.string  # type: ignore
                assert isinstance(main_title, str)

            # Convert the page
            webpage_text = f"# {main_title}\n\n" + _CustomMarkdownify().convert_soup(
                body_elm
            )
        else:
            webpage_text = _CustomMarkdownify().convert_soup(soup)

        return DocumentConverterResult(
            title=main_title,
            text_content=webpage_text,
        )


class YouTubeConverter(DocumentConverter):
    """Handle YouTube specially, focusing on the video title, description, and transcript."""

    def convert(
        self, local_path: str, **kwargs: Any
    ) -> Union[None, DocumentConverterResult]:
        # Bail if not YouTube
        extension = kwargs.get("file_extension", "")
        if extension.lower() not in [".html", ".htm"]:
            return None
        url = kwargs.get("url", "")
        if not url.startswith("https://www.youtube.com/watch?"):
            return None

        # Parse the file
        soup = None
        with open(local_path, "rt", encoding="utf-8") as fh:
            soup = BeautifulSoup(fh.read(), "html.parser")

        # Read the meta tags
        assert soup.title is not None and soup.title.string is not None
        metadata: Dict[str, str] = {"title": soup.title.string}
        for meta in soup(["meta"]):
            for a in meta.attrs:
                if a in ["itemprop", "property", "name"]:
                    metadata[meta[a]] = meta.get("content", "")
                    break

        # We can also try to read the full description. This is more prone to breaking, since it reaches into the page implementation
        try:
            for script in soup(["script"]):
                content = script.text
                if "ytInitialData" in content:
                    lines = re.split(r"\r?\n", content)
                    obj_start = lines[0].find("{")
                    obj_end = lines[0].rfind("}")
                    if obj_start >= 0 and obj_end >= 0:
                        data = json.loads(lines[0][obj_start : obj_end + 1])
                        attrdesc = self._findKey(
                            data, "attributedDescriptionBodyText"
                        )  # type: ignore
                        if attrdesc:
                            metadata["description"] = str(attrdesc["content"])
                    break
        except Exception:
            pass

        # Start preparing the page
        webpage_text = "# YouTube\n"

        title = self._get(metadata, ["title", "og:title", "name"])  # type: ignore
        assert isinstance(title, str)

        if title:
            webpage_text += f"\n## {title}\n"

        stats = ""
        views = self._get(metadata, ["interactionCount"])  # type: ignore
        if views:
            stats += f"- **Views:** {views}\n"

        keywords = self._get(metadata, ["keywords"])  # type: ignore
        if keywords:
            stats += f"- **Keywords:** {keywords}\n"

        runtime = self._get(metadata, ["duration"])  # type: ignore
        if runtime:
            stats += f"- **Runtime:** {runtime}\n"

        if len(stats) > 0:
            webpage_text += f"\n### Video Metadata\n{stats}\n"

        description = self._get(
            metadata, ["description", "og:description"]
        )  # type: ignore
        if description:
            webpage_text += f"\n### Description\n{description}\n"

        if IS_YOUTUBE_TRANSCRIPT_CAPABLE:
            transcript_text = ""
            parsed_url = urlparse(url)  # type: ignore
            params = parse_qs(parsed_url.query)  # type: ignore
            if "v" in params:
                assert isinstance(params["v"][0], str)
                video_id = str(params["v"][0])
                try:
                    # Must be a single transcript.
                    transcript = YouTubeTranscriptApi.get_transcript(
                        video_id
                    )  # type: ignore
                    transcript_text = " ".join(
                        [part["text"] for part in transcript]
                    )  # type: ignore
                    # Alternative formatting:
                    # formatter = TextFormatter()
                    # formatter.format_transcript(transcript)
                except Exception:
                    pass
            if transcript_text:
                webpage_text += f"\n### Transcript\n{transcript_text}\n"

        title = title if title else soup.title.string
        assert isinstance(title, str)

        return DocumentConverterResult(
            title=title,
            text_content=webpage_text,
        )

    def _get(
        self,
        metadata: Dict[str, str],
        keys: List[str],
        default: Union[str, None] = None,
    ) -> Union[str, None]:
        for k in keys:
            if k in metadata:
                return metadata[k]
        return default

    # TODO: Fix json type
    def _findKey(self, json: Any, key: str) -> Union[str, None]:
        if isinstance(json, list):
            for elm in json:
                ret = self._findKey(elm, key)
                if ret is not None:
                    return ret
        elif isinstance(json, dict):
            for k in json:
                if k == key:
                    return json[k]
                else:
                    ret = self._findKey(json[k], key)
                    if ret is not None:
                        return ret
        return None


class BingSerpConverter(DocumentConverter):
    """
    Handle Bing results pages (only the organic search results).
    NOTE: It is better to use the Bing API
    """

    def convert(self, local_path, **kwargs) -> Union[None, DocumentConverterResult]:
        # Bail if not a Bing SERP
        extension = kwargs.get("file_extension", "")
        if extension.lower() not in [".html", ".htm"]:
            return None
        url = kwargs.get("url", "")
        if not re.search(r"^https://www\.bing\.com/search\?q=", url):
            return None

        # Parse the query parameters
        parsed_params = parse_qs(urlparse(url).query)
        query = parsed_params.get("q", [""])[0]

        # Parse the file
        soup = None
        with open(local_path, "rt", encoding="utf-8") as fh:
            soup = BeautifulSoup(fh.read(), "html.parser")

        # Clean up some formatting
        for tptt in soup.find_all(class_="tptt"):
            if hasattr(tptt, "string") and tptt.string:
                tptt.string += " "
        for slug in soup.find_all(class_="algoSlug_icon"):
            slug.extract()

        # Parse the algorithmic results
        _markdownify = _CustomMarkdownify()
        results = list()
        for result in soup.find_all(class_="b_algo"):
            # Rewrite redirect urls
            for a in result.find_all("a", href=True):
                parsed_href = urlparse(a["href"])
                qs = parse_qs(parsed_href.query)

                # The destination is contained in the u parameter,
                # but appears to be base64 encoded, with some prefix
                if "u" in qs:
                    u = (
                        qs["u"][0][2:].strip() + "=="
                    )  # Python 3 doesn't care about extra padding

                    try:
                        # RFC 4648 / Base64URL" variant, which uses "-" and "_"
                        a["href"] = base64.b64decode(u, altchars="-_").decode("utf-8")
                    except UnicodeDecodeError:
                        pass
                    except binascii.Error:
                        pass

            # Convert to markdown
            md_result = _markdownify.convert_soup(result).strip()
            lines = [line.strip() for line in re.split(r"\n+", md_result)]
            results.append("\n".join([line for line in lines if len(line) > 0]))

        webpage_text = (
            f"## A Bing search for '{query}' found the following results:\n\n"
            + "\n\n".join(results)
        )

        return DocumentConverterResult(
            title=None if soup.title is None else soup.title.string,
            text_content=webpage_text,
        )


class PdfConverter(DocumentConverter):
    """
    Converts PDFs to Markdown with support for extracting and including images.
    """

    def __init__(self, llm=None, product_mode="lite"):
        super().__init__()
        self.llm = llm
        self.product_mode = product_mode

    def convert(self, local_path, **kwargs) -> Union[None, DocumentConverterResult]:
        # Bail if not a PDF
        extension = kwargs.get("file_extension", "")
        if extension.lower() != ".pdf":
            return None
        image_output_dir = None
        if kwargs.get("image_output_dir", None):
            image_output_dir = kwargs.get("image_output_dir")
        else:
            # Create output directory for images if it doesn't exist
            image_output_dir = os.path.join(
                os.path.dirname(local_path),
                "_images",
                os.path.basename(local_path).replace(" ", "_"),
            )
        os.makedirs(image_output_dir, exist_ok=True)

        text_content = []
        image_count = 0

        # Open and process PDF
        with open(local_path, "rb") as file:
            # Create PDF parser and document
            parser = PDFParser(file)
            document = PDFDocument(parser)
            rsrcmgr = PDFResourceManager()
            laparams = LAParams()
            device = PDFPageAggregator(rsrcmgr, laparams=laparams)
            interpreter = PDFPageInterpreter(rsrcmgr, device)

            # Process each page
            for page in PDFPage.create_pages(document):
                interpreter.process_page(page)
                layout = device.get_result()

                # Extract text and images from the page
                page_content = self._process_layout(
                    layout, image_output_dir, image_count
                )

                text_content.extend(page_content)
                image_count += len([c for c in page_content if c.startswith("![Image")])

        return DocumentConverterResult(
            title=None,
            text_content="\n".join(text_content),
        )

    def _process_layout(
        self, layout, image_output_dir: str, image_count: int
    ) -> List[str]:
        """Process the layout of a PDF page, extracting both text and images."""
        content = []
        local_image_count = image_count
        for lt_obj in layout:
            # Handle images
            if isinstance(lt_obj, LTImage) or (
                isinstance(lt_obj, LTFigure) and lt_obj.name.startswith("Im")
            ):
                # 将整个图片处理逻辑包裹在 try-except 中，确保图片解析失败不影响文本解析
                try:
                    image_data = None
                    image_meta = {}
                    # 添加uuid后缀避免文件名冲突
                    unique_id = str(uuid.uuid4())[:8]
                    image_path = os.path.join(
                        image_output_dir, f"image_{local_image_count}_{unique_id}.png"
                    )

                    if hasattr(lt_obj, "stream"):
                        image_data = lt_obj.stream.get_data()
                        image_meta = lt_obj.stream.attrs
                    elif hasattr(lt_obj, "filter"):
                        image_data = lt_obj.filter

                    if image_data:
                        if isinstance(lt_obj, LTImage):
                            try:
                                iw = ImageWriter(image_output_dir)
                                name = iw.export_image(lt_obj)
                                suffix = os.path.splitext(name)[1]
                                temp_path = os.path.join(image_output_dir, name)
                                # 添加uuid后缀避免文件名冲突
                                unique_id = str(uuid.uuid4())[:8]
                                image_path = os.path.join(
                                    image_output_dir,
                                    f"image_{local_image_count}_{unique_id}{suffix}",
                                )
                                os.rename(temp_path, image_path)
                                content.append(
                                    f"![Image {local_image_count}]({image_path})"
                                )
                                # ===== 修改：通过FilterRuleManager单例实例判断是否需要解析图片
                                v = try_parse_image(image_path, self.llm)
                                if v:
                                    content.append("<image_content>")
                                    content.append(v)
                                    content.append("</image_content>")
                                # =====
                                local_image_count += 1
                                continue
                            except Exception as e:
                                logger.warning(
                                    f"Error exporting LTImage: {e}, trying alternative methods"
                                )
                        try:
                            # Try to handle raw pixel data
                            if "BitsPerComponent" in image_meta:
                                width = image_meta["Width"]
                                height = image_meta["Height"]
                                bits = image_meta["BitsPerComponent"]
                                colorspace = image_meta["ColorSpace"].name
                                new_image_data = np.frombuffer(
                                    image_data, dtype=np.uint8
                                )
                                # Normalize to 8-bit if necessary
                                if bits != 8:
                                    max_val = (1 << bits) - 1
                                    new_image_data = (
                                        new_image_data.astype("float32") * 255 / max_val
                                    ).astype("uint8")

                                if colorspace == "DeviceRGB":
                                    new_image_data = new_image_data.reshape(
                                        (height, width, 3)
                                    )
                                    img = Image.fromarray(new_image_data, "RGB")
                                    # 添加uuid后缀避免文件名冲突
                                    unique_id = str(uuid.uuid4())[:8]
                                    image_path = os.path.join(
                                        image_output_dir,
                                        f"image_{local_image_count}_{unique_id}.png",
                                    )
                                    img.save(image_path)
                                    content.append(
                                        f"![Image {local_image_count}]({image_path})\n"
                                    )
                                    v = try_parse_image(image_path, self.llm)
                                    if v:
                                        content.append("<image_content>")
                                        content.append(v)
                                        content.append("</image_content>")
                                    local_image_count += 1
                                    continue
                                elif colorspace == "DeviceGray":
                                    new_image_data = new_image_data.reshape(
                                        (height, width)
                                    )
                                    img = Image.fromarray(new_image_data, "L")
                                    # 添加uuid后缀避免文件名冲突
                                    unique_id = str(uuid.uuid4())[:8]
                                    image_path = os.path.join(
                                        image_output_dir,
                                        f"image_{local_image_count}_{unique_id}.png",
                                    )
                                    img.save(image_path)
                                    content.append(
                                        f"![Image {local_image_count}]({image_path})\n"
                                    )
                                    v = try_parse_image(image_path, self.llm)
                                    if v:
                                        content.append("<image_content>")
                                        content.append(v)
                                        content.append("</image_content>")
                                    local_image_count += 1
                                    continue
                        except Exception as e:
                            logger.warning(
                                f"Error extracting image from pixel data: {e}, fallback to writing original data"
                            )

                        # 尝试直接写入原始数据
                        try:
                            # 添加uuid后缀避免文件名冲突
                            unique_id = str(uuid.uuid4())[:8]
                            image_path = os.path.join(
                                image_output_dir,
                                f"image_{local_image_count}_{unique_id}.png",
                            )
                            with open(image_path, "wb") as img_file:
                                img_file.write(image_data)

                            content.append(
                                f"![Image {local_image_count}]({image_path})\n"
                            )
                            # ===== 新增：图片解析
                            v = try_parse_image(image_path, self.llm)
                            if v:
                                content.append("<image_content>")
                                content.append(v)
                                content.append("</image_content>")
                            local_image_count += 1
                        except Exception as e:
                            logger.warning(f"Error writing raw image data: {e}")

                except Exception as e:
                    # 图片处理完全失败，记录日志但继续处理其他元素
                    logger.warning(
                        f"Failed to extract image from PDF: {e}, skipping this image"
                    )

            # Handle text
            if hasattr(lt_obj, "get_text"):
                text = lt_obj.get_text().strip()
                if text:
                    content.append(text)

            # Recursively process nested layouts
            elif hasattr(lt_obj, "_objs"):
                content.extend(
                    self._process_layout(lt_obj._objs, image_output_dir, image_count)
                )

        return content


class DocxConverter(HtmlConverter):
    """
    Converts DOCX files to Markdown. Style information (e.g.m headings) and tables are preserved where possible.
    """

    def __init__(self):
        self._image_counter = 0
        super().__init__()

    def _save_image(self, image, output_dir: str) -> str:
        """
        保存图片并返回相对路径，使用递增的计数器来命名文件
        """
        # 获取图片内容和格式
        image_format = (
            image.content_type.split("/")[-1] if image.content_type else "png"
        )

        # 增加计数器并生成文件名
        self._image_counter += 1
        # 添加uuid后缀避免文件名冲突
        unique_id = str(uuid.uuid4())[:8]
        image_filename = f"image_{self._image_counter}_{unique_id}.{image_format}"

        # 保存图片
        image_path = os.path.join(output_dir, image_filename)
        with image.open() as image_content, open(image_path, "wb") as f:
            f.write(image_content.read())

        return image_path

    def convert(self, local_path, **kwargs) -> Union[None, DocumentConverterResult]:
        # Bail if not a DOCX
        extension = kwargs.get("file_extension", "")
        if extension.lower() != ".docx":
            return None

        # 设置图片输出目录
        image_output_dir = None
        if kwargs.get("image_output_dir", None):
            image_output_dir = kwargs.get("image_output_dir")
        else:
            # Create output directory for images if it doesn't exist
            image_output_dir = os.path.join(
                os.path.dirname(local_path),
                "_images",
                os.path.basename(local_path).replace(" ", "_"),
            )
        os.makedirs(image_output_dir, exist_ok=True)

        result = None
        with open(local_path, "rb") as docx_file:
            # 配置图片转换器
            def transform_image(image):
                return {
                    "src": self._save_image(image, image_output_dir),
                    "alt": (
                        image.alt_text
                        if image.alt_text
                        else f"Image {self._image_counter}"
                    ),
                }

            # 进行转换
            result = mammoth.convert_to_html(
                docx_file, convert_image=mammoth.images.inline(transform_image)
            )
            html_content = result.value
            result = self._convert(html_content)

        return result


class XlsxConverter(HtmlConverter):
    """
    Converts XLSX files to Markdown, with each sheet presented as a separate Markdown table.
    """

    def convert(self, local_path, **kwargs) -> Union[None, DocumentConverterResult]:
        # Bail if not a XLSX
        extension = kwargs.get("file_extension", "")
        if extension.lower() != ".xlsx":
            return None

        sheets = pd.read_excel(local_path, sheet_name=None)
        md_content = ""
        for s in sheets:
            md_content += f"## {s}\n"
            html_content = sheets[s].to_html(index=False)
            md_content += self._convert(html_content).text_content.strip() + "\n\n"

        return DocumentConverterResult(
            title=None,
            text_content=md_content.strip(),
        )


class PptxConverter(HtmlConverter):
    """
    Converts PPTX files to Markdown. Supports heading, tables and images with alt text.
    """

    def convert(self, local_path, **kwargs) -> Union[None, DocumentConverterResult]:
        # Bail if not a PPTX
        extension = kwargs.get("file_extension", "")
        if extension.lower() != ".pptx":
            return None

        md_content = ""

        presentation = pptx.Presentation(local_path)
        slide_num = 0
        for slide in presentation.slides:
            slide_num += 1

            md_content += f"\n\n<!-- Slide number: {slide_num} -->\n"

            title = slide.shapes.title
            for shape in slide.shapes:
                # Pictures
                if self._is_picture(shape):
                    # https://github.com/scanny/python-pptx/pull/512#issuecomment-1713100069
                    alt_text = ""
                    try:
                        alt_text = shape._element._nvXxPr.cNvPr.attrib.get("descr", "")
                    except Exception:
                        pass

                    # A placeholder name
                    filename = re.sub(r"\W", "", shape.name) + ".jpg"
                    md_content += (
                        "\n!["
                        + (alt_text if alt_text else shape.name)
                        + "]("
                        + filename
                        + ")\n"
                    )

                # Tables
                if self._is_table(shape):
                    html_table = "<html><body><table>"
                    first_row = True
                    for row in shape.table.rows:
                        html_table += "<tr>"
                        for cell in row.cells:
                            if first_row:
                                html_table += "<th>" + html.escape(cell.text) + "</th>"
                            else:
                                html_table += "<td>" + html.escape(cell.text) + "</td>"
                        html_table += "</tr>"
                        first_row = False
                    html_table += "</table></body></html>"
                    md_content += (
                        "\n" + self._convert(html_table).text_content.strip() + "\n"
                    )

                # Text areas
                elif shape.has_text_frame:
                    if shape == title:
                        md_content += "# " + shape.text.lstrip() + "\n"
                    else:
                        md_content += shape.text + "\n"

            md_content = md_content.strip()

            if slide.has_notes_slide:
                md_content += "\n\n### Notes:\n"
                notes_frame = slide.notes_slide.notes_text_frame
                if notes_frame is not None:
                    md_content += notes_frame.text
                md_content = md_content.strip()

        return DocumentConverterResult(
            title=None,
            text_content=md_content.strip(),
        )

    def _is_picture(self, shape):
        if shape.shape_type == pptx.enum.shapes.MSO_SHAPE_TYPE.PICTURE:
            return True
        if shape.shape_type == pptx.enum.shapes.MSO_SHAPE_TYPE.PLACEHOLDER:
            if hasattr(shape, "image"):
                return True
        return False

    def _is_table(self, shape):
        if shape.shape_type == pptx.enum.shapes.MSO_SHAPE_TYPE.TABLE:
            return True
        return False


class MediaConverter(DocumentConverter):
    """
    Abstract class for multi-modal media (e.g., images and audio)
    """

    def _get_metadata(self, local_path):
        exiftool = shutil.which("exiftool")
        if not exiftool:
            return None
        else:
            try:
                result = subprocess.run(
                    [exiftool, "-json", local_path], capture_output=True, text=True
                ).stdout
                return json.loads(result)[0]
            except Exception:
                return None


class WavConverter(MediaConverter):
    """
    Converts WAV files to markdown via extraction of metadata (if `exiftool` is installed), and speech transcription (if `speech_recognition` is installed).
    """

    def convert(self, local_path, **kwargs) -> Union[None, DocumentConverterResult]:
        # Bail if not a XLSX
        extension = kwargs.get("file_extension", "")
        if extension.lower() != ".wav":
            return None

        md_content = ""

        # Add metadata
        metadata = self._get_metadata(local_path)
        if metadata:
            for f in [
                "Title",
                "Artist",
                "Author",
                "Band",
                "Album",
                "Genre",
                "Track",
                "DateTimeOriginal",
                "CreateDate",
                "Duration",
            ]:
                if f in metadata:
                    md_content += f"{f}: {metadata[f]}\n"

        # Transcribe
        if IS_AUDIO_TRANSCRIPTION_CAPABLE:
            try:
                transcript = self._transcribe_audio(local_path)
                md_content += "\n\n### Audio Transcript:\n" + (
                    "[No speech detected]" if transcript == "" else transcript
                )
            except Exception:
                md_content += (
                    "\n\n### Audio Transcript:\nError. Could not transcribe this audio."
                )

        return DocumentConverterResult(
            title=None,
            text_content=md_content.strip(),
        )

    def _transcribe_audio(self, local_path) -> str:
        recognizer = sr.Recognizer()
        with sr.AudioFile(local_path) as source:
            audio = recognizer.record(source)
            return recognizer.recognize_google(audio).strip()


class Mp3Converter(WavConverter):
    """
    Converts MP3 files to markdown via extraction of metadata (if `exiftool` is installed), and speech transcription (if `speech_recognition` AND `pydub` are installed).
    """

    def convert(self, local_path, **kwargs) -> Union[None, DocumentConverterResult]:
        # Bail if not a MP3
        extension = kwargs.get("file_extension", "")
        if extension.lower() != ".mp3":
            return None

        md_content = ""

        # Add metadata
        metadata = self._get_metadata(local_path)
        if metadata:
            for f in [
                "Title",
                "Artist",
                "Author",
                "Band",
                "Album",
                "Genre",
                "Track",
                "DateTimeOriginal",
                "CreateDate",
                "Duration",
            ]:
                if f in metadata:
                    md_content += f"{f}: {metadata[f]}\n"

        # Transcribe
        if IS_AUDIO_TRANSCRIPTION_CAPABLE:
            handle, temp_path = tempfile.mkstemp(suffix=".wav")
            os.close(handle)
            try:
                sound = pydub.AudioSegment.from_mp3(local_path)
                sound.export(temp_path, format="wav")

                _args = dict()
                _args.update(kwargs)
                _args["file_extension"] = ".wav"

                try:
                    transcript = super()._transcribe_audio(temp_path).strip()
                    md_content += "\n\n### Audio Transcript:\n" + (
                        "[No speech detected]" if transcript == "" else transcript
                    )
                except Exception:
                    md_content += "\n\n### Audio Transcript:\nError. Could not transcribe this audio."

            finally:
                os.unlink(temp_path)

        # Return the result
        return DocumentConverterResult(
            title=None,
            text_content=md_content.strip(),
        )


class ImageConverter(MediaConverter):
    """
    Converts images to markdown via extraction of metadata (if `exiftool` is installed), OCR (if `easyocr` is installed), and description via a multimodal LLM (if an mlm_client is configured).
    """

    def convert(self, local_path, **kwargs) -> Union[None, DocumentConverterResult]:
        # Bail if not a XLSX
        extension = kwargs.get("file_extension", "")
        if extension.lower() not in [".jpg", ".jpeg", ".png"]:
            return None

        md_content = ""

        # Add metadata
        metadata = self._get_metadata(local_path)
        if metadata:
            for f in [
                "ImageSize",
                "Title",
                "Caption",
                "Description",
                "Keywords",
                "Artist",
                "Author",
                "DateTimeOriginal",
                "CreateDate",
                "GPSPosition",
            ]:
                if f in metadata:
                    md_content += f"{f}: {metadata[f]}\n"

        # Try describing the image with GPTV
        mlm_client = kwargs.get("mlm_client")
        mlm_model = kwargs.get("mlm_model")
        if mlm_client is not None and mlm_model is not None:
            md_content += (
                "\n# Description:\n"
                + self._get_mlm_description(
                    local_path,
                    extension,
                    mlm_client,
                    mlm_model,
                    prompt=kwargs.get("mlm_prompt"),
                ).strip()
                + "\n"
            )

        return DocumentConverterResult(
            title=None,
            text_content=md_content,
        )

    def _get_mlm_description(self, local_path, extension, client, model, prompt=None):
        if prompt is None or prompt.strip() == "":
            prompt = "Write a detailed caption for this image."

        sys.stderr.write(f"MLM Prompt:\n{prompt}\n")

        data_uri = ""
        with open(local_path, "rb") as image_file:
            content_type, encoding = mimetypes.guess_type("_dummy" + extension)
            if content_type is None:
                content_type = "image/jpeg"
            image_base64 = base64.b64encode(image_file.read()).decode("utf-8")
            data_uri = f"data:{content_type};base64,{image_base64}"

        messages = [
            {
                "role": "user",
                "content": [
                    {"type": "text", "text": prompt},
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": data_uri,
                        },
                    },
                ],
            }
        ]

        response = client.chat.completions.create(model=model, messages=messages)
        return response.choices[0].message.content


class FileConversionException(BaseException):
    pass


class UnsupportedFormatException(BaseException):
    pass


class MarkItDown:
    """(In preview) An extremely simple text-based document reader, suitable for LLM use.
    This reader will convert common file-types or webpages to Markdown."""

    def __init__(
        self,
        requests_session: Optional[requests.Session] = None,
        mlm_client: Optional[Any] = None,
        mlm_model: Optional[Any] = None,
        llm: Optional[Any] = None,
        product_mode: Optional[str] = None,
    ):
        # 初始化FilterRuleManager单例实例
        self._filter_rule_manager = FilterRuleManager.get_instance()
        if requests_session is None:
            self._requests_session = requests.Session()
        else:
            self._requests_session = requests_session

        self._mlm_client = mlm_client
        self._mlm_model = mlm_model

        # 新增：保存llm和product_mode
        self._llm = llm
        self._product_mode = product_mode

        self._page_converters: List[DocumentConverter] = []

        # Register converters for successful browsing operations
        # Later registrations are tried first / take higher priority than earlier registrations
        # To this end, the most specific converters should appear below the most generic converters
        self.register_page_converter(PlainTextConverter())
        self.register_page_converter(HtmlConverter())
        self.register_page_converter(WikipediaConverter())
        self.register_page_converter(YouTubeConverter())
        self.register_page_converter(BingSerpConverter())
        self.register_page_converter(DocxConverter())
        self.register_page_converter(XlsxConverter())
        self.register_page_converter(PptxConverter())
        self.register_page_converter(WavConverter())
        self.register_page_converter(Mp3Converter())
        self.register_page_converter(ImageConverter())
        self.register_page_converter(PdfConverter(llm, product_mode))

    def convert(
        self, source: Union[str, requests.Response], **kwargs: Any
    ) -> DocumentConverterResult:  # TODO: deal with kwargs
        """
        Args:
            - source: can be a string representing a path or url, or a requests.response object
            - extension: specifies the file extension to use when interpreting the file. If None, infer from source (path, uri, content-type, etc.)
        """
        # Local path or url
        if isinstance(source, str):
            if (
                source.startswith("http://")
                or source.startswith("https://")
                or source.startswith("file://")
            ):
                return self.convert_url(source, **kwargs)
            else:
                return self.convert_local(source, **kwargs)
        # Request response
        elif isinstance(source, requests.Response):
            return self.convert_response(source, **kwargs)

    def convert_local(
        self, path: str, **kwargs: Any
    ) -> DocumentConverterResult:  # TODO: deal with kwargs
        # Prepare a list of extensions to try (in order of priority)
        ext = kwargs.get("file_extension")
        extensions = [ext] if ext is not None else []

        # Get extension alternatives from the path and puremagic
        base, ext = os.path.splitext(path)
        self._append_ext(extensions, ext)

        if not extensions:
            for g in self._guess_ext_magic(path):
                self._append_ext(extensions, g)

        # Convert
        return self._convert(path, extensions, **kwargs)

    # TODO what should stream's type be?
    def convert_stream(
        self, stream: Any, **kwargs: Any
    ) -> DocumentConverterResult:  # TODO: deal with kwargs
        # Prepare a list of extensions to try (in order of priority)
        ext = kwargs.get("file_extension")
        extensions = [ext] if ext is not None else []

        # Save the file locally to a temporary file. It will be deleted before this method exits
        handle, temp_path = tempfile.mkstemp()
        fh = os.fdopen(handle, "wb")
        result = None
        try:
            # Write to the temporary file
            content = stream.read()
            if isinstance(content, str):
                fh.write(content.encode("utf-8"))
            else:
                fh.write(content)
            fh.close()

            # Use puremagic to check for more extension options
            for g in self._guess_ext_magic(temp_path):
                self._append_ext(extensions, g)

            # Convert
            result = self._convert(temp_path, extensions, **kwargs)
        # Clean up
        finally:
            try:
                fh.close()
            except Exception:
                pass
            os.unlink(temp_path)

        return result

    def convert_url(
        self, url: str, **kwargs: Any
    ) -> DocumentConverterResult:  # TODO: fix kwargs type
        # Send a HTTP request to the URL
        response = self._requests_session.get(url, stream=True)
        response.raise_for_status()
        return self.convert_response(response, **kwargs)

    def convert_response(
        self, response: requests.Response, **kwargs: Any
    ) -> DocumentConverterResult:  # TODO fix kwargs type
        # Prepare a list of extensions to try (in order of priority)
        ext = kwargs.get("file_extension")
        extensions = [ext] if ext is not None else []

        # Guess from the mimetype
        content_type = response.headers.get("content-type", "").split(";")[0]
        self._append_ext(extensions, mimetypes.guess_extension(content_type))

        # Read the content disposition if there is one
        content_disposition = response.headers.get("content-disposition", "")
        m = re.search(r"filename=([^;]+)", content_disposition)
        if m:
            base, ext = os.path.splitext(m.group(1).strip("\"'"))
            self._append_ext(extensions, ext)

        # Read from the extension from the path
        base, ext = os.path.splitext(urlparse(response.url).path)
        self._append_ext(extensions, ext)

        # Save the file locally to a temporary file. It will be deleted before this method exits
        handle, temp_path = tempfile.mkstemp()
        fh = os.fdopen(handle, "wb")
        result = None
        try:
            # Download the file
            for chunk in response.iter_content(chunk_size=512):
                fh.write(chunk)
            fh.close()

            # Use puremagic to check for more extension options
            for g in self._guess_ext_magic(temp_path):
                self._append_ext(extensions, g)

            # Convert
            result = self._convert(temp_path, extensions, url=response.url)
        # Clean up
        finally:
            try:
                fh.close()
            except Exception:
                pass
            os.unlink(temp_path)

        return result

    def _convert(
        self, local_path: str, extensions: List[Union[str, None]], **kwargs
    ) -> DocumentConverterResult:
        error_trace = ""
        res = None
        for ext in extensions + [None]:  # Try last with no extension
            for converter in self._page_converters:
                _kwargs = copy.deepcopy(kwargs)

                # Overwrite file_extension appropriately
                if ext is None:
                    if "file_extension" in _kwargs:
                        del _kwargs["file_extension"]
                else:
                    _kwargs.update({"file_extension": ext})

                # Copy any additional global options
                if "mlm_client" not in _kwargs and self._mlm_client is not None:
                    _kwargs["mlm_client"] = self._mlm_client

                if "mlm_model" not in _kwargs and self._mlm_model is not None:
                    _kwargs["mlm_model"] = self._mlm_model

                # If we hit an error log it and keep trying
                try:
                    res = converter.convert(local_path, **_kwargs)
                except Exception:
                    error_trace = ("\n\n" + traceback.format_exc()).strip()

                if res is not None:
                    # Normalize the content
                    res.text_content = "\n".join(
                        [line.rstrip() for line in re.split(r"\r?\n", res.text_content)]
                    )
                    res.text_content = re.sub(r"\n{3,}", "\n\n", res.text_content)

                    # Todo
                    return res

        # If we got this far without success, report any exceptions
        if len(error_trace) > 0:
            raise FileConversionException(
                f"Could not convert '{local_path}' to Markdown. File type was recognized as {extensions}. While converting the file, the following error was encountered:\n\n{error_trace}"
            )

        # Nothing can handle it!
        raise UnsupportedFormatException(
            f"Could not convert '{local_path}' to Markdown. The formats {extensions} are not supported."
        )

    def _append_ext(self, extensions, ext):
        """Append a unique non-None, non-empty extension to a list of extensions."""
        if ext is None:
            return
        ext = ext.strip()
        if ext == "":
            return
        # if ext not in extensions:
        if True:
            extensions.append(ext)

    def _guess_ext_magic(self, path):
        """Use puremagic (a Python implementation of libmagic) to guess a file's extension based on the first few bytes."""
        # Use puremagic to guess
        try:
            guesses = puremagic.magic_file(path)
            extensions = list()
            for g in guesses:
                ext = g.extension.strip()
                if len(ext) > 0:
                    if not ext.startswith("."):
                        ext = "." + ext
                    if ext not in extensions:
                        extensions.append(ext)
            return extensions
        except FileNotFoundError:
            pass
        except IsADirectoryError:
            pass
        except PermissionError:
            pass
        return []

    def register_page_converter(self, converter: DocumentConverter) -> None:
        """Register a page text converter."""
        self._page_converters.insert(0, converter)


def try_parse_image(image_path: str, llm=None):
    """
    根据FilterRuleManager单例实例判断是否需要解析图片，如果需要则调用ImageLoader.image_to_markdown。
    解析失败会自动捕获异常。
    """
    import uuid

    start_time = time.time()
    req_id = str(uuid.uuid4())[:8]
    logger.info(f"\n==== [try_parse_image] START | req_id={req_id} ====")
    logger.info(f"[try_parse_image][{req_id}] image_path: {image_path}, llm: {llm}")
    if FilterRuleManager.get_instance().should_parse_image(image_path):
        logger.info(
            f"[try_parse_image][{req_id}] should_parse_image=True, start parsing..."
        )
        try:
            v = ImageLoader.image_to_markdown(image_path, llm=llm, engine="paddle")
            logger.info(
                f"[try_parse_image][{req_id}] image_to_markdown result: {str(v)[:200]}"
            )

            if not v:
                return ""

            if llm:
                v = ImageLoader.format_table_in_content(v, llm)
                logger.info(
                    f"[try_parse_image][{req_id}] format_table_in_content result: {str(v)[:200]}"
                )
            elapsed = time.time() - start_time
            logger.info(
                f"[try_parse_image][{req_id}] SUCCESS | execution time: {elapsed:.3f} seconds"
            )
            logger.info(f"==== [try_parse_image] END | req_id={req_id} ====")
            return v
        except Exception as e:
            elapsed = time.time() - start_time
            logger.error(
                f"[try_parse_image][{req_id}] EXCEPTION | execution time: {elapsed:.3f} seconds | image_path: {image_path} | llm: {llm}"
            )
            logger.exception(e)
            logger.info(
                f"==== [try_parse_image] END (EXCEPTION) | req_id={req_id} ===="
            )
            return ""
    else:
        logger.info(
            f"[try_parse_image][{req_id}] should_parse_image=False, skip parsing."
        )
        logger.info(f"==== [try_parse_image] END (SKIP) | req_id={req_id} ====")
        return ""



"""引导模块 - 初始化系统并启动应用"""

import asyncio
import os
from pathlib import Path
from autocoder.auto_coder_runner import (
    load_tokenizer,
    initialize_system,
    InitializeSystemRequest,
    configure,
    start as start_engine,
    completer,
    ask,
    coding,
    chat,
    design,
    voice_input,
    auto_command,
    run_agentic,
    execute_shell_command,
    active_context,
    get_mcp_server,
    stop as stop_engine,
)
from autocoder.common.core_config import (
    cycle_mode,
    get_mode,
    set_mode,
    toggle_human_as_model,
    get_human_as_model_string,
    toggle_agentic_mode,
    get_agentic_mode_string,
)
from autocoder.plugins import PluginManager
from autocoder.chat_auto_coder_lang import (
    get_message,
    get_message_with_format as get_message_with_format_local,
)
from autocoder.terminal.args import parse_arguments
from autocoder.terminal.help import show_help
from autocoder.terminal.app import TerminalApp


def load_builtin_plugins(plugin_manager):
    """加载内置插件"""
    try:
        # 发现所有可用的插件
        discovered_plugins = plugin_manager.discover_plugins()

        # 排除的示例插件列表
        excluded_plugins = {
            "autocoder.plugins.dynamic_completion_example",
            "autocoder.plugins.sample_plugin",
        }

        # 自动加载内置插件（在autocoder.plugins模块中的插件）
        builtin_loaded = 0
        for plugin_class in discovered_plugins:
            module_name = plugin_class.__module__
            if (
                module_name.startswith("autocoder.plugins.")
                and not module_name.endswith(".__init__")
                and module_name not in excluded_plugins
            ):
                try:
                    if plugin_manager.load_plugin(plugin_class):
                        builtin_loaded += 1
                        print(f"✓ Loaded builtin plugin: {plugin_class.plugin_name()}")
                except Exception as e:
                    print(
                        f"✗ Failed to load builtin plugin {plugin_class.plugin_name()}: {e}"
                    )

        if builtin_loaded > 0:
            print(
                get_message_with_format_local(
                    "loaded_plugins_builtin", count=builtin_loaded
                )
            )
        else:
            print(get_message("no_builtin_plugins_loaded"))

    except Exception as e:
        print(f"Error loading builtin plugins: {e}")


def run_cli():
    """CLI 入口函数"""
    # 加载 tokenizer
    load_tokenizer()

    # 解析参数
    args = parse_arguments()

    # 处理 lite/pro 模式
    if args.lite:
        args.product_mode = "lite"
    if args.pro:
        args.product_mode = "pro"

    # 初始化系统（如果不是 quick 模式）
    if not args.quick:
        initialize_system(
            InitializeSystemRequest(
                product_mode=args.product_mode,
                skip_provider_selection=args.skip_provider_selection,
                debug=args.debug,
                quick=args.quick,
                lite=args.lite,
                pro=args.pro,
            )
        )

    # 启动引擎
    start_engine()

    # 初始化插件系统
    plugin_manager = PluginManager()
    plugin_manager.load_global_plugin_dirs()

    # 添加内置插件目录
    plugins_dir = Path(__file__).parent.parent / "plugins"
    plugin_manager.add_global_plugin_directory(str(plugins_dir))

    # 加载保存的运行时配置
    plugin_manager.load_runtime_cfg()

    # 自动加载内置插件
    load_builtin_plugins(plugin_manager)

    # 配置 product_mode
    configure(f"product_mode:{args.product_mode}")

    # 创建原始函数字典
    original_functions = {
        "ask": ask,
        "coding": coding,
        "chat": chat,
        "design": design,
        "voice_input": voice_input,
        "auto_command": auto_command,
        "run_agentic": run_agentic,
        "execute_shell_command": execute_shell_command,
        "active_context": active_context,
    }

    # 创建 wrapped functions
    wrapped_functions = {}
    for func_name, original_func in original_functions.items():
        wrapped_functions[func_name] = plugin_manager.wrap_function(
            original_func, func_name
        )

    # 创建 show_help 函数
    def show_help_func():
        show_help(plugin_manager)

    # 创建终端应用
    app = TerminalApp(
        plugin_manager=plugin_manager,
        wrapped_functions=wrapped_functions,
        configure_func=configure,
        show_help_func=show_help_func,
        base_completer=completer,
        get_mode_func=get_mode,
        get_human_as_model_string_func=get_human_as_model_string,
        get_agentic_mode_string_func=get_agentic_mode_string,
        cycle_mode_func=cycle_mode,
        toggle_human_as_model_func=toggle_human_as_model,
        toggle_agentic_mode_func=toggle_agentic_mode,
        voice_input_func=wrapped_functions.get("voice_input"),
        get_mcp_server_func=get_mcp_server,
        stop_engine_func=stop_engine,
        debug=args.debug,
    )

    # 运行应用
    asyncio.run(app.run())



import pydantic
from typing import List

class IndexItem(pydantic.BaseModel):
    module_name: str
    symbols: str
    last_modified: float
    md5: str  # 新增文件内容的MD5哈希值字段


class TargetFile(pydantic.BaseModel):
    file_path: str
    reason: str = pydantic.Field(
        ..., description="The reason why the file is the target file"
    )


class VerifyFileRelevance(pydantic.BaseModel):
    relevant_score: int
    reason: str


class FileList(pydantic.BaseModel):
    file_list: List[TargetFile]

class FileNumberList(pydantic.BaseModel):
    file_list: List[int]    



"""
Conversation Normalizer

This module provides tools to normalize conversation structures by ensuring
user/assistant role pairs are properly balanced. It offers two strategies:
merge and delete for handling unpaired messages.
"""

from typing import List, Dict, Any, Optional, Tuple
from enum import Enum
from pydantic import BaseModel
from loguru import logger
from autocoder.common.international.message_manager import get_message


class NormalizationStrategy(Enum):
    """规整化策略枚举"""
    MERGE = "merge"      # 合并连续的相同角色消息
    DELETE = "delete"    # 删除多余的消息，保留第一个


class NormalizationResult(BaseModel):
    """规整化结果"""
    normalized_conversations: List[Dict[str, Any]]
    original_count: int
    normalized_count: int
    changes_made: List[Dict[str, Any]]
    strategy_used: str
    
    
class ConversationNormalizer:
    """
    会话规整化工具类
    
    检查conversations内部user/assistant角色是否成对出现，
    如果不是成对出现，提供两种策略：
    1. merge: 合并连续的相同角色消息
    2. delete: 删除多余的消息，只保留第一个
    
    示例：
    输入: [user, user, assistant] 
    - merge策略: [merged_user, assistant]
    - delete策略: [first_user, assistant]
    """
    
    def __init__(self, default_strategy: NormalizationStrategy = NormalizationStrategy.MERGE):
        """
        初始化规整化器
        
        Args:
            default_strategy: 默认策略
        """
        self.default_strategy = default_strategy
        
    def normalize_conversations(
        self, 
        conversations: List[Dict[str, Any]], 
        strategy: Optional[NormalizationStrategy] = None
    ) -> NormalizationResult:
        """
        规整化会话列表
        
        Args:
            conversations: 原始会话列表
            strategy: 规整化策略，如果为None则使用默认策略
            
        Returns:
            NormalizationResult: 规整化结果
        """
        if strategy is None:
            strategy = self.default_strategy
            
        logger.info(f"开始规整化会话，策略: {strategy.value}, 原始消息数: {len(conversations)}")
        
        # 验证输入
        if not conversations:
            return NormalizationResult(
                normalized_conversations=[],
                original_count=0,
                normalized_count=0,
                changes_made=[],
                strategy_used=strategy.value
            )
        
        # 执行规整化
        if strategy == NormalizationStrategy.MERGE:
            result = self._merge_strategy(conversations)
        elif strategy == NormalizationStrategy.DELETE:
            result = self._delete_strategy(conversations)
        else:
            raise ValueError(f"不支持的策略: {strategy}")
            
        logger.info(f"规整化完成，原始: {result.original_count}, 规整化后: {result.normalized_count}, 变更: {len(result.changes_made)}")
        
        return result
    
    def _merge_strategy(self, conversations: List[Dict[str, Any]]) -> NormalizationResult:
        """
        合并策略实现
        
        连续的相同角色消息会被合并成一个消息
        """
        normalized = []
        changes_made = []
        
        i = 0
        while i < len(conversations):
            current_msg = conversations[i].copy()
            current_role = current_msg.get("role", "")
            
            # 查找连续的相同角色消息
            consecutive_messages = [current_msg]
            j = i + 1
            
            while j < len(conversations) and conversations[j].get("role", "") == current_role:
                consecutive_messages.append(conversations[j])
                j += 1
            
            # 如果有多个连续消息，进行合并
            if len(consecutive_messages) > 1:
                merged_content = self._merge_message_contents(consecutive_messages)
                merged_msg = {
                    "role": current_role,
                    "content": merged_content
                }
                
                # 保留其他字段（如果有的话）
                for key, value in current_msg.items():
                    if key not in ["role", "content"]:
                        merged_msg[key] = value
                
                normalized.append(merged_msg)
                
                # 记录变更
                changes_made.append({
                    "action": "merge",
                    "original_indices": list(range(i, j)),
                    "original_messages": consecutive_messages,
                    "merged_message": merged_msg,
                    "merged_count": len(consecutive_messages)
                })
            else:
                # 单个消息直接添加
                normalized.append(current_msg)
            
            i = j
        
        return NormalizationResult(
            normalized_conversations=normalized,
            original_count=len(conversations),
            normalized_count=len(normalized),
            changes_made=changes_made,
            strategy_used=NormalizationStrategy.MERGE.value
        )
    
    def _delete_strategy(self, conversations: List[Dict[str, Any]]) -> NormalizationResult:
        """
        删除策略实现
        
        连续的相同角色消息只保留第一个，删除其余的
        """
        normalized = []
        changes_made = []
        
        i = 0
        while i < len(conversations):
            current_msg = conversations[i].copy()
            current_role = current_msg.get("role", "")
            
            # 添加第一个消息
            normalized.append(current_msg)
            
            # 查找连续的相同角色消息
            consecutive_messages = [current_msg]
            j = i + 1
            
            while j < len(conversations) and conversations[j].get("role", "") == current_role:
                consecutive_messages.append(conversations[j])
                j += 1
            
            # 如果有多个连续消息，记录删除操作
            if len(consecutive_messages) > 1:
                deleted_messages = consecutive_messages[1:]  # 除了第一个之外的所有消息
                
                changes_made.append({
                    "action": "delete",
                    "kept_index": i,
                    "kept_message": current_msg,
                    "deleted_indices": list(range(i + 1, j)),
                    "deleted_messages": deleted_messages,
                    "deleted_count": len(deleted_messages)
                })
            
            i = j
        
        return NormalizationResult(
            normalized_conversations=normalized,
            original_count=len(conversations),
            normalized_count=len(normalized),
            changes_made=changes_made,
            strategy_used=NormalizationStrategy.DELETE.value
        )
    
    def _merge_message_contents(self, messages: List[Dict[str, Any]]) -> str:
        """
        合并多个消息的内容
        
        Args:
            messages: 要合并的消息列表
            
        Returns:
            合并后的内容字符串
        """
        contents = []
        
        for i, msg in enumerate(messages):
            content = msg.get("content", "")
            if content.strip():
                # 为每个消息添加序号标识
                if len(messages) > 1:
                    contents.append(f"[消息 {i+1}] {content}")
                else:
                    contents.append(content)
        
        return "\n\n".join(contents)
    
    def analyze_conversation_structure(self, conversations: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        分析会话结构
        
        Args:
            conversations: 会话列表
            
        Returns:
            结构分析结果
        """
        if not conversations:
            return {
                "total_messages": 0,
                "role_distribution": {},
                "consecutive_groups": [],
                "needs_normalization": False,
                "issues": []
            }
        
        role_distribution = {}
        consecutive_groups = []
        issues = []
        
        # 统计角色分布
        for msg in conversations:
            role = msg.get("role", "unknown")
            role_distribution[role] = role_distribution.get(role, 0) + 1
        
        # 分析连续组
        current_group = {"role": conversations[0].get("role", ""), "count": 1, "start_index": 0}
        
        for i in range(1, len(conversations)):
            role = conversations[i].get("role", "")
            if role == current_group["role"]:
                current_group["count"] += 1
            else:
                if current_group["count"] > 1:
                    consecutive_groups.append(current_group.copy())
                current_group = {"role": role, "count": 1, "start_index": i}
        
        # 添加最后一组
        if current_group["count"] > 1:
            consecutive_groups.append(current_group)
        
        # 检查是否需要规整化
        needs_normalization = len(consecutive_groups) > 0
        
        # 收集问题
        if needs_normalization:
            for group in consecutive_groups:
                issues.append(f"连续的 {group['count']} 个 '{group['role']}' 消息从索引 {group['start_index']} 开始")
        
        # 检查是否以user开始
        if conversations and conversations[0].get("role") != "user":
            issues.append(f"会话不是以 'user' 角色开始，而是 '{conversations[0].get('role')}'")
        
        # 检查是否以assistant结束
        if conversations and conversations[-1].get("role") != "assistant":
            issues.append(f"会话不是以 'assistant' 角色结束，而是 '{conversations[-1].get('role')}'")
        
        return {
            "total_messages": len(conversations),
            "role_distribution": role_distribution,
            "consecutive_groups": consecutive_groups,
            "needs_normalization": needs_normalization,
            "issues": issues
        }
    
    def validate_normalized_conversation(self, conversations: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        验证规整化后的会话是否符合预期
        
        Args:
            conversations: 规整化后的会话列表
            
        Returns:
            验证结果
        """
        validation_result = {
            "is_valid": True,
            "errors": [],
            "warnings": [],
            "structure_info": {}
        }
        
        if not conversations:
            validation_result["warnings"].append("会话列表为空")
            return validation_result
        
        # 检查是否还有连续的相同角色
        for i in range(len(conversations) - 1):
            current_role = conversations[i].get("role", "")
            next_role = conversations[i + 1].get("role", "")
            
            if current_role == next_role:
                validation_result["is_valid"] = False
                validation_result["errors"].append(f"索引 {i} 和 {i+1} 处仍有连续的 '{current_role}' 角色")
        
        # 检查角色交替模式
        expected_pattern = True
        for i in range(len(conversations)):
            role = conversations[i].get("role", "")
            if i % 2 == 0:  # 偶数索引应该是user
                if role != "user":
                    expected_pattern = False
                    validation_result["warnings"].append(f"索引 {i} 期望 'user' 但得到 '{role}'")
            else:  # 奇数索引应该是assistant
                if role != "assistant":
                    expected_pattern = False
                    validation_result["warnings"].append(f"索引 {i} 期望 'assistant' 但得到 '{role}'")
        
        validation_result["structure_info"] = {
            "follows_user_assistant_pattern": expected_pattern,
            "total_messages": len(conversations),
            "starts_with_user": conversations[0].get("role") == "user" if conversations else False,
            "ends_with_assistant": conversations[-1].get("role") == "assistant" if conversations else False
        }
        
        return validation_result




"""
GitHub API 提供者实现
"""
from typing import List, Dict, Any
from loguru import logger

from ..base_provider import BasePlatformProvider
from ..models import RepoInfo, PRData, PRResult, PRInfo
from ..utils import build_pr_url


class GitHubProvider(BasePlatformProvider):
    """GitHub API 提供者"""
    
    def _get_auth_header(self) -> str:
        """获取认证头"""
        return f"token {self.config.token}"
    
    def create_pr(self, repo_info: RepoInfo, pr_data: PRData) -> PRResult:
        """创建 Pull Request"""
        self._validate_repo_info(repo_info)
        self._validate_pr_data(pr_data)
        
        url = f"{self.config.base_url}/repos/{repo_info.full_name}/pulls"
        
        payload = {
            "title": pr_data.title,
            "body": pr_data.description,
            "head": pr_data.source_branch,
            "base": pr_data.target_branch,
            "draft": pr_data.draft or self.config.draft,
            "maintainer_can_modify": self.config.maintainer_can_modify
        }
        
        try:
            response = self._make_request('POST', url, data=payload)
            data = response.json()
            
            pr_url = build_pr_url(self.config.platform, repo_info, data['number'])
            
            result = PRResult(
                success=True,
                pr_number=data['number'],
                pr_url=pr_url,
                pr_id=str(data['id']),
                platform=self.config.platform,
                raw_response=data
            )
            
            logger.info(f"GitHub PR 创建成功: {pr_url}")
            return result
            
        except Exception as e:
            logger.error(f"创建 GitHub PR 失败: {e}")
            return PRResult(
                success=False,
                error_message=str(e),
                platform=self.config.platform
            )
    
    def get_pr(self, repo_info: RepoInfo, pr_number: int) -> PRResult:
        """获取 PR 信息"""
        url = f"{self.config.base_url}/repos/{repo_info.full_name}/pulls/{pr_number}"
        
        try:
            response = self._make_request('GET', url)
            data = response.json()
            
            pr_info = PRInfo(
                number=data['number'],
                title=data['title'],
                description=data.get('body', ''),
                state=data['state'],
                source_branch=data['head']['ref'],
                target_branch=data['base']['ref'],
                author=data['user']['login'],
                created_at=data['created_at'],
                updated_at=data['updated_at'],
                merged_at=data.get('merged_at'),
                pr_url=data['html_url'],
                labels=[label['name'] for label in data.get('labels', [])],
                assignees=[assignee['login'] for assignee in data.get('assignees', [])],
                mergeable=data.get('mergeable'),
                draft=data.get('draft', False),
                raw_data=data
            )
            
            return PRResult(
                success=True,
                pr_number=pr_info.number,
                pr_url=pr_info.pr_url,
                platform=self.config.platform,
                raw_response=data
            )
            
        except Exception as e:
            logger.error(f"获取 GitHub PR 失败: {e}")
            return PRResult(
                success=False,
                error_message=str(e),
                platform=self.config.platform
            )
    
    def update_pr(self, repo_info: RepoInfo, pr_number: int, **kwargs) -> PRResult:
        """更新 PR"""
        url = f"{self.config.base_url}/repos/{repo_info.full_name}/pulls/{pr_number}"
        
        payload = {}
        if 'title' in kwargs:
            payload['title'] = kwargs['title']
        if 'description' in kwargs:
            payload['body'] = kwargs['description']
        if 'state' in kwargs:
            payload['state'] = kwargs['state']
        
        try:
            response = self._make_request('PATCH', url, data=payload)
            data = response.json()
            
            return PRResult(
                success=True,
                pr_number=data['number'],
                pr_url=data['html_url'],
                platform=self.config.platform,
                raw_response=data
            )
            
        except Exception as e:
            logger.error(f"更新 GitHub PR 失败: {e}")
            return PRResult(
                success=False,
                error_message=str(e),
                platform=self.config.platform
            )
    
    def close_pr(self, repo_info: RepoInfo, pr_number: int) -> PRResult:
        """关闭 PR"""
        return self.update_pr(repo_info, pr_number, state='closed')
    
    def merge_pr(self, repo_info: RepoInfo, pr_number: int, **kwargs) -> PRResult:
        """合并 PR"""
        url = f"{self.config.base_url}/repos/{repo_info.full_name}/pulls/{pr_number}/merge"
        
        payload = {
            'commit_title': kwargs.get('commit_title', ''),
            'commit_message': kwargs.get('commit_message', ''),
            'merge_method': kwargs.get('merge_method', 'merge')
        }
        
        try:
            response = self._make_request('PUT', url, data=payload)
            data = response.json()
            
            return PRResult(
                success=True,
                pr_number=pr_number,
                platform=self.config.platform,
                raw_response=data
            )
            
        except Exception as e:
            logger.error(f"合并 GitHub PR 失败: {e}")
            return PRResult(
                success=False,
                error_message=str(e),
                platform=self.config.platform
            )
    
    def list_prs(
        self, 
        repo_info: RepoInfo, 
        state: str = "open",
        per_page: int = 30,
        page: int = 1
    ) -> List[PRInfo]:
        """列出仓库的PR"""
        url = f"{self.config.base_url}/repos/{repo_info.full_name}/pulls"
        params = {
            'state': state,
            'per_page': str(per_page),
            'page': str(page)
        }
        
        try:
            response = self._make_request('GET', url, params=params)
            data = response.json()
            
            prs = []
            for pr_data in data:
                pr_info = PRInfo(
                    number=pr_data['number'],
                    title=pr_data['title'],
                    description=pr_data.get('body', ''),
                    state=pr_data['state'],
                    source_branch=pr_data['head']['ref'],
                    target_branch=pr_data['base']['ref'],
                    author=pr_data['user']['login'],
                    created_at=pr_data['created_at'],
                    updated_at=pr_data['updated_at'],
                    merged_at=pr_data.get('merged_at'),
                    pr_url=pr_data['html_url'],
                    labels=[label['name'] for label in pr_data.get('labels', [])],
                    assignees=[assignee['login'] for assignee in pr_data.get('assignees', [])],
                    mergeable=pr_data.get('mergeable'),
                    draft=pr_data.get('draft', False),
                    raw_data=pr_data
                )
                prs.append(pr_info)
            
            return prs
            
        except Exception as e:
            logger.error(f"列出 GitHub PR 失败: {e}")
            return []


"""
Configuration merge utilities for global and project scope merging.

This module provides utilities to deep merge configuration dictionaries,
with project-level configuration taking precedence over global configuration.
"""

from typing import Any, Dict
from pathlib import Path
import os


def get_global_config_dir() -> Path:
    """
    Get the global configuration directory path.

    Returns:
        Path: ~/.auto-coder/plugins/chat-auto-coder
    """
    home = Path.home()
    return home / ".auto-coder" / "plugins" / "chat-auto-coder"


def deep_merge_config(
    global_conf: Dict[str, Any], project_conf: Dict[str, Any]
) -> Dict[str, Any]:
    """
    Deep merge two configuration dictionaries with project config taking precedence.

    Merge strategy:
    - Scalars (str/int/bool/float/None): Project value overrides global value
    - Dicts: Recursively merge, with project keys overriding global keys
    - Lists: Project list completely replaces global list (no merging)
    - Type conflicts: Project value takes precedence

    Args:
        global_conf: Global configuration dictionary
        project_conf: Project configuration dictionary

    Returns:
        Merged configuration with project values taking precedence

    Examples:
        >>> global_c = {"a": 1, "b": {"x": 10, "y": 20}, "c": [1, 2]}
        >>> project_c = {"b": {"x": 100}, "d": 4}
        >>> deep_merge_config(global_c, project_c)
        {'a': 1, 'b': {'x': 100, 'y': 20}, 'c': [1, 2], 'd': 4}
    """
    if not isinstance(global_conf, dict):
        global_conf = {}
    if not isinstance(project_conf, dict):
        project_conf = {}

    # Start with a copy of global config
    result = global_conf.copy()

    # Merge project config into result
    for key, project_value in project_conf.items():
        if key not in result:
            # Key only exists in project, add it directly
            result[key] = project_value
        else:
            global_value = result[key]

            # If both are dicts, recursively merge
            if isinstance(global_value, dict) and isinstance(project_value, dict):
                result[key] = deep_merge_config(global_value, project_value)
            else:
                # For all other types (scalars, lists, type conflicts), project wins
                result[key] = project_value

    return result



















"""对话缓冲区管理"""

import os
from typing import List, Optional
from .message import ChatMessage
from autocoder.common.v2.agent.agentic_edit_types import (
    ConversationIdEvent,
    TokenUsageEvent,
    WindowLengthChangeEvent,
    LLMThinkingEvent,
    LLMOutputEvent,
    ToolCallEvent,
    ToolResultEvent,
    PlanModeRespondEvent,
    PreCommitEvent,
    CommitEvent,
    PullRequestEvent,
    CompletionEvent,
    ErrorEvent,
)


class ConversationBuffer:
    """对话缓冲区，类似聊天应用的消息列表"""

    def __init__(self, max_messages: int = 200):
        self.max_messages = max_messages
        self.messages: List[ChatMessage] = []
        self.welcome_messages: List[ChatMessage] = []
        self.token_stats = {"input_tokens": 0, "output_tokens": 0, "model": ""}

        # 强类型属性定义
        self._thinking_buffer: Optional[str] = None
        self._current_response: Optional[str] = None

        self._init_welcome_messages()

    def _init_welcome_messages(self):
        """初始化欢迎消息"""
        self.welcome_messages.append(
            ChatMessage("✦ Welcome to Auto-Coder.Chat!", "welcome")
        )
        self.welcome_messages.append(ChatMessage("", "welcome"))
        self.welcome_messages.append(
            ChatMessage("/help for help, /status to view current setup", "welcome")
        )
        self.welcome_messages.append(ChatMessage("", "welcome"))
        cwd = os.getcwd()
        self.welcome_messages.append(ChatMessage(f"cwd: {cwd}", "welcome"))
        self.welcome_messages.append(ChatMessage("", "welcome"))

    def get_welcome_text(self):
        """获取欢迎消息的格式化文本"""
        result = []
        for msg in self.welcome_messages:
            result.extend(msg.to_formatted_text())
        return result

    def add_user_message(self, content: str):
        """添加用户消息"""
        self.messages.append(ChatMessage(content, "user"))
        self._trim_messages()

    def add_assistant_message(self, content: str):
        """添加助手消息"""
        self.messages.append(ChatMessage(content, "assistant"))
        self._trim_messages()

    def add_system_message(self, content: str, ansi: bool = False):
        """添加系统消息"""
        self.messages.append(ChatMessage(content, "system", ansi=ansi))
        self._trim_messages()

    def add_event(self, event):
        """处理 SdkRunner 事件 - 参考 terminal_runner.py 的格式化方式"""
        if isinstance(event, ConversationIdEvent):
            # 对话ID事件静默处理，不显示
            pass

        elif isinstance(event, TokenUsageEvent):
            meta = event.usage
            self.token_stats["input_tokens"] += meta.input_tokens_count
            self.token_stats["output_tokens"] += meta.generated_tokens_count
            # SingleOutputMeta 没有 model_name 属性，保持原有逻辑
            # 模型名称通过其他方式获取

        elif isinstance(event, WindowLengthChangeEvent):
            # 显示 token 使用情况
            if event.tokens_used > event.pruned_tokens_used:
                self.add_system_message(
                    f"Conversation tokens: {event.tokens_used} -> {event.pruned_tokens_used} "
                    f"(round: {event.conversation_round})"
                )

        elif isinstance(event, LLMThinkingEvent):
            # 实时显示思考过程（灰色显示）
            if self._thinking_buffer is not None:
                self._thinking_buffer += event.text
                # 更新最后一条思考消息
                if self.messages and self.messages[-1]._is_thinking:
                    self.messages[-1].content = self._thinking_buffer
                else:
                    msg = ChatMessage(self._thinking_buffer, "system")
                    msg._is_thinking = True
                    self.messages.append(msg)
            else:
                self._thinking_buffer = event.text
                msg = ChatMessage(self._thinking_buffer, "system")
                msg._is_thinking = True
                self.messages.append(msg)

        elif isinstance(event, LLMOutputEvent):
            # 清除思考缓冲区（如果有）
            if self._thinking_buffer is not None:
                self._thinking_buffer = None

            # LLM 输出直接添加为助手消息（流式更新）
            if self._current_response is not None:
                self._current_response += event.text
                if self.messages and self.messages[-1].role == "assistant":
                    self.messages[-1].content = self._current_response
                else:
                    self.messages.append(
                        ChatMessage(self._current_response, "assistant")
                    )
            else:
                self._current_response = event.text
                self.messages.append(ChatMessage(self._current_response, "assistant"))

        elif isinstance(event, ToolCallEvent):
            # 使用简洁的工具调用显示
            from .tool_display import get_tool_simple_display

            tool_display = get_tool_simple_display(event.tool)
            self.add_system_message(f"🔧 {tool_display}")

        elif isinstance(event, ToolResultEvent):
            # 显示工具结果（简洁版本）
            from .tool_display import get_tool_result_display

            result = event.result
            result_display = get_tool_result_display(
                event.tool_name,
                result.success,
                result.message if not result.success else None,
            )
            self.add_system_message(result_display)

        elif isinstance(event, PlanModeRespondEvent):
            try:
                response_text = event.completion.response
                if response_text:
                    self.add_assistant_message(response_text)
            except Exception:
                pass

        elif isinstance(event, PreCommitEvent):
            if event.commit_result:
                self.add_system_message("📦 Changes staged")

        elif isinstance(event, CommitEvent):
            if event.commit_result and event.commit_result.success:
                self.add_system_message(
                    f"✅ Committed: {event.commit_result.commit_hash[:8]}"
                )

        elif isinstance(event, PullRequestEvent):
            if event.pull_request_result and event.pull_request_result.success:
                self.add_system_message(
                    f"🎉 PR #{event.pull_request_result.pr_number} created"
                )

        elif isinstance(event, CompletionEvent):
            # 清除思考和响应缓冲区
            if self._thinking_buffer is not None:
                self._thinking_buffer = None

            if self._current_response is not None:
                if self.messages and self.messages[-1].role == "assistant":
                    self.messages[-1].content = self._current_response
                else:
                    self.add_assistant_message(self._current_response)
                self._current_response = None
            else:
                try:
                    result_text = event.completion.result
                    if result_text:
                        self.add_assistant_message(result_text)
                except Exception:
                    pass

        elif isinstance(event, ErrorEvent):
            self.add_system_message(f"❌ Error: {event.message}")

    def _trim_messages(self):
        """保持消息数量在限制内"""
        if len(self.messages) > self.max_messages:
            self.messages = self.messages[-self.max_messages :]

    def get_formatted_text(self):
        """获取格式化的文本用于显示"""
        if not self.messages:
            return [("", "")]

        result = []
        for message in self.messages:
            result.extend(message.to_formatted_text())
        return result

    def clear_conversation(self):
        """清空对话历史"""
        self.messages = []
        self._current_response = None
        self._thinking_buffer = None



import pytest
from .tool_content_detector import ToolContentDetector


class TestToolContentDetector:
    """测试 ToolContentDetector 类的各种功能"""

    def setup_method(self):
        """每个测试方法前的设置"""
        self.detector = ToolContentDetector()

    def test_init(self):
        """测试初始化"""
        detector = ToolContentDetector()
        assert detector.replacement_message == "Content cleared to save tokens"
        assert len(detector.supported_tools) == 2
        assert "write_to_file" in detector.supported_tools
        assert "replace_in_file" in detector.supported_tools

    def test_init_with_custom_message(self):
        """测试自定义替换消息的初始化"""
        custom_message = "Custom replacement message"
        detector = ToolContentDetector(replacement_message=custom_message)
        assert detector.replacement_message == custom_message

    def test_detect_tool_call_write_to_file(self):
        """测试检测 write_to_file 工具调用"""
        content = """Some text before
<write_to_file>
<path>test.py</path>
<content>
def hello():
    print("Hello, World!")
</content>
</write_to_file>
Some text after"""

        result = self.detector.detect_tool_call(content)
        assert result is not None
        assert result['tool_name'] == 'write_to_file'
        assert 'def hello():' in result['content_inner']
        assert result['start_pos'] > 0
        assert result['end_pos'] > result['start_pos']

    def test_detect_tool_call_replace_in_file(self):
        """测试检测 replace_in_file 工具调用"""
        content = """Some text before
<replace_in_file>
<path>test.py</path>
<diff>
- old_function()
+ new_function()
</diff>
</replace_in_file>
Some text after"""

        result = self.detector.detect_tool_call(content)
        assert result is not None
        assert result['tool_name'] == 'replace_in_file'
        assert 'old_function()' in result['content_inner']
        assert 'new_function()' in result['content_inner']

    def test_detect_tool_call_case_insensitive(self):
        """测试大小写不敏感的工具调用检测"""
        content = """
<WRITE_TO_FILE>
<path>test.py</path>
<CONTENT>
print("test")
</CONTENT>
</WRITE_TO_FILE>"""

        result = self.detector.detect_tool_call(content)
        assert result is not None
        assert result['tool_name'] == 'write_to_file'

    def test_detect_tool_call_no_tool(self):
        """测试不包含工具调用的内容"""
        content = "This is just plain text without any tool calls."
        result = self.detector.detect_tool_call(content)
        assert result is None

    def test_detect_tool_call_empty_content(self):
        """测试空内容"""
        result = self.detector.detect_tool_call("")
        assert result is None

        result = self.detector.detect_tool_call(None)
        assert result is None

    def test_detect_tool_call_incomplete_tool(self):
        """测试不完整的工具调用"""
        content = "<write_to_file><path>test.py</path>"
        result = self.detector.detect_tool_call(content)
        assert result is None

    def test_replace_tool_content_short_content(self):
        """测试替换短内容（不应该替换）"""
        content = """
<write_to_file>
<path>test.py</path>
<content>
print("short")
</content>
</write_to_file>"""

        new_content, replaced = self.detector.replace_tool_content(content, max_content_length=500)
        assert not replaced
        assert new_content == content

    def test_replace_tool_content_long_content(self):
        """测试替换长内容"""
        long_code = "def test():\n    " + "print('line')\n    " * 50
        content = f"""
<write_to_file>
<path>test.py</path>
<content>
{long_code}
</content>
</write_to_file>"""

        new_content, replaced = self.detector.replace_tool_content(content, max_content_length=100)
        assert replaced
        assert "Content cleared to save tokens" in new_content
        assert long_code not in new_content
        assert "<write_to_file>" in new_content
        assert "</write_to_file>" in new_content

    def test_replace_tool_content_replace_in_file(self):
        """测试替换 replace_in_file 中的长内容"""
        long_diff = "- old_line\n+ new_line\n" * 50
        content = f"""
<replace_in_file>
<path>test.py</path>
<diff>
{long_diff}
</diff>
</replace_in_file>"""

        new_content, replaced = self.detector.replace_tool_content(content, max_content_length=100)
        assert replaced
        assert "Content cleared to save tokens" in new_content
        assert long_diff not in new_content
        assert "<diff>" in new_content
        assert "</diff>" in new_content

    def test_replace_tool_content_no_tool(self):
        """测试替换不包含工具调用的内容"""
        content = "This is just plain text."
        new_content, replaced = self.detector.replace_tool_content(content)
        assert not replaced
        assert new_content == content

    def test_get_tool_content_stats_with_tool(self):
        """测试获取包含工具调用的统计信息"""
        content = """Some prefix
<write_to_file>
<path>test.py</path>
<content>
def hello():
    print("Hello!")
</content>
</write_to_file>
Some suffix"""

        stats = self.detector.get_tool_content_stats(content)
        assert stats['has_tool_call'] is True
        assert stats['tool_name'] == 'write_to_file'
        assert stats['total_length'] == len(content)
        assert stats['tool_content_length'] > 0
        assert stats['prefix_length'] > 0
        assert stats['suffix_length'] > 0

    def test_get_tool_content_stats_no_tool(self):
        """测试获取不包含工具调用的统计信息"""
        content = "This is just plain text."
        stats = self.detector.get_tool_content_stats(content)
        assert stats['has_tool_call'] is False
        assert stats['total_length'] == len(content)

    def test_get_tool_content_stats_empty(self):
        """测试获取空内容的统计信息"""
        stats = self.detector.get_tool_content_stats("")
        assert stats['has_tool_call'] is False
        assert stats['total_length'] == 0

    def test_is_tool_call_content_true(self):
        """测试判断包含工具调用的内容"""
        content = """
<write_to_file>
<path>test.py</path>
<content>print("test")</content>
</write_to_file>"""

        assert self.detector.is_tool_call_content(content) is True

    def test_is_tool_call_content_false(self):
        """测试判断不包含工具调用的内容"""
        content = "This is just plain text."
        assert self.detector.is_tool_call_content(content) is False

    def test_extract_tool_content_write_to_file(self):
        """测试提取 write_to_file 的内容"""
        content = """
<write_to_file>
<path>test.py</path>
<content>
def hello():
    print("Hello!")
</content>
</write_to_file>"""

        extracted = self.detector.extract_tool_content(content)
        assert extracted is not None
        assert "def hello():" in extracted
        assert 'print("Hello!")' in extracted

    def test_extract_tool_content_replace_in_file(self):
        """测试提取 replace_in_file 的内容"""
        content = """
<replace_in_file>
<path>test.py</path>
<diff>
- old_function()
+ new_function()
</diff>
</replace_in_file>"""

        extracted = self.detector.extract_tool_content(content)
        assert extracted is not None
        assert "old_function()" in extracted
        assert "new_function()" in extracted

    def test_extract_tool_content_no_tool(self):
        """测试提取不包含工具调用的内容"""
        content = "This is just plain text."
        extracted = self.detector.extract_tool_content(content)
        assert extracted is None

    def test_get_supported_tools(self):
        """测试获取支持的工具列表"""
        tools = self.detector.get_supported_tools()
        assert isinstance(tools, list)
        assert len(tools) == 2
        assert "write_to_file" in tools
        assert "replace_in_file" in tools

    def test_multiple_tool_calls(self):
        """测试包含多个工具调用的内容（应该只检测第一个）"""
        content = """
<write_to_file>
<path>test1.py</path>
<content>print("first")</content>
</write_to_file>

<write_to_file>
<path>test2.py</path>
<content>print("second")</content>
</write_to_file>"""

        result = self.detector.detect_tool_call(content)
        assert result is not None
        assert result['tool_name'] == 'write_to_file'
        assert 'print("first")' in result['content_inner']

    def test_nested_tags(self):
        """测试嵌套标签的处理"""
        content = """
<write_to_file>
<path>test.py</path>
<content>
<div>
    <p>This is HTML content</p>
</div>
</content>
</write_to_file>"""

        result = self.detector.detect_tool_call(content)
        assert result is not None
        assert "<div>" in result['content_inner']
        assert "<p>This is HTML content</p>" in result['content_inner']

    def test_multiline_content(self):
        """测试多行内容的处理"""
        content = """
<write_to_file>
<path>test.py</path>
<content>
class TestClass:
    def __init__(self):
        self.value = 42
    
    def method(self):
        return self.value
</content>
</write_to_file>"""

        result = self.detector.detect_tool_call(content)
        assert result is not None
        assert "class TestClass:" in result['content_inner']
        assert "def __init__(self):" in result['content_inner']
        assert "def method(self):" in result['content_inner']

    def test_content_with_special_characters(self):
        """测试包含特殊字符的内容"""
        content = """
<write_to_file>
<path>test.py</path>
<content>
# This is a comment with special chars: !@#$%^&*()
print("String with 'quotes' and \"double quotes\"")
regex = r"[a-zA-Z0-9]+"
</content>
</write_to_file>"""

        result = self.detector.detect_tool_call(content)
        assert result is not None
        assert "!@#$%^&*()" in result['content_inner']
        assert "'quotes'" in result['content_inner']  # 修复：查找单引号包围的quotes
        assert r'[a-zA-Z0-9]+' in result['content_inner']


if __name__ == "__main__":
    pytest.main([__file__, "-v"]) 


"""
MCP Tools Server Module

This module provides the McpServer class for handling MCP server operations
in a thread-safe manner. It manages requests and responses through async queues.
"""

import asyncio
from asyncio import Queue as AsyncQueue
import threading
import signal
import atexit
from typing import Union
from loguru import logger

from .hub import McpHub, MCP_BUILD_IN_SERVERS
from .executor import McpExecutor
from .installer import McpServerInstaller
from .types import (
    McpRequest, McpInstallRequest, McpRemoveRequest, McpListRequest,
    McpListRunningRequest, McpRefreshRequest, McpServerInfoRequest,
    McpResponse, ServerInfo, InstallResult, RemoveResult, ListResult,
    ListRunningResult, RefreshResult, QueryResult, ErrorResult,
    StringResult, MarketplaceAddRequest, MarketplaceAddResult,
    MarketplaceUpdateRequest, MarketplaceUpdateResult, MarketplaceMCPServerItem
)
from autocoder.utils.llms import get_single_llm
from autocoder.chat_auto_coder_lang import get_message_with_format


class McpServer:
    """
    MCP Server handling class that manages MCP operations in a thread-safe manner.
    
    This class provides an async interface for MCP operations while maintaining
    thread safety through proper queue management.
    """
    
    def __init__(self):
        self._request_queue = AsyncQueue()
        self._response_queue = AsyncQueue()
        self._running = False
        self._task = None
        self._loop = None
        self._thread = None
        self._installer = McpServerInstaller()
        self._shutdown_event = threading.Event()

    def start(self):
        """Start the MCP server in a separate thread"""
        if self._running:
            return

        self._running = True
        self._shutdown_event.clear()
        self._loop = asyncio.new_event_loop()
        self._thread = threading.Thread(target=self._run_event_loop, daemon=True)
        self._thread.start()

    def stop(self):
        """Stop the MCP server"""
        if not self._running:
            return
            
        self._running = False
        self._shutdown_event.set()
        
        if self._loop and self._loop.is_running():
            # Cancel the task gracefully
            if self._task and not self._task.done():
                self._loop.call_soon_threadsafe(self._task.cancel)
            
            # Send None to break the request processing loop
            try:
                future = asyncio.run_coroutine_threadsafe(
                    self._request_queue.put(None), self._loop
                )
                future.result(timeout=1.0)
            except Exception:
                pass
            
            # Stop the event loop
            self._loop.call_soon_threadsafe(self._loop.stop)
        
        # Wait for thread to finish
        if self._thread and self._thread.is_alive():
            self._thread.join(timeout=2.0)
            if self._thread.is_alive():
                logger.warning("MCP server thread did not stop gracefully")

    def _run_event_loop(self):
        """Run the event loop in the dedicated thread"""
        if self._loop is None:
            return
            
        asyncio.set_event_loop(self._loop)
        
        try:
            self._task = self._loop.create_task(self._process_request())
            self._loop.run_forever()
        except KeyboardInterrupt:
            logger.info("KeyboardInterrupt received in MCP server event loop")
        except Exception as e:
            logger.error(f"Error in MCP server event loop: {e}")
        finally:
            # Clean up
            if self._task and not self._task.done():
                self._task.cancel()
                try:
                    self._loop.run_until_complete(self._task)
                except asyncio.CancelledError:
                    pass
            
            # Close the loop
            try:
                self._loop.close()
            except Exception as e:
                logger.error(f"Error closing event loop: {e}")

    async def _process_request(self):
        """
        Process incoming requests and generate responses
        """
        hub = McpHub()
        
        try:
            # May block if there are abnormal contents in mcp settings.json
            await hub.initialize()
        except Exception as e:
            logger.error(f"Failed to initialize MCP hub: {e}")
            return

        while self._running:
            try:
                # Use timeout to allow checking _running flag
                try:
                    request = await asyncio.wait_for(
                        self._request_queue.get(), timeout=1.0
                    )
                except asyncio.TimeoutError:
                    continue
                    
                if request is None:
                    break

                if isinstance(request, McpInstallRequest):
                    response = await self._installer.install_server(request, hub)
                    await self._response_queue.put(response)

                elif isinstance(request, McpRemoveRequest):
                    try:
                        await hub.remove_server_config(request.server_name)
                        await self._response_queue.put(McpResponse(
                            result=get_message_with_format("mcp_remove_success", result=request.server_name),
                            raw_result=RemoveResult(
                                success=True,
                                server_name=request.server_name
                            )
                        ))
                    except Exception as e:
                        await self._response_queue.put(McpResponse(
                            result="", 
                            error=get_message_with_format("mcp_remove_error", error=str(e)),
                            raw_result=RemoveResult(
                                success=False,
                                server_name=request.server_name,
                                error=str(e)
                            )
                        ))

                elif isinstance(request, McpListRequest):
                    try:
                        # Get built-in servers
                        builtin_servers = []
                        for name, config in MCP_BUILD_IN_SERVERS.items():
                            marketplace_item = MarketplaceMCPServerItem(
                                name=name,
                                description=f"Built-in: {name}",
                                mcp_type="command",
                                command=config.get("command") or "",
                                args=config.get("args", []),
                                env=config.get("env", {})
                            )
                            builtin_servers.append(marketplace_item)

                        # Get external servers
                        external_servers = self._installer.get_mcp_external_servers()
                        external_items = []
                        for server in external_servers:
                            marketplace_item = MarketplaceMCPServerItem(
                                name=server.name,
                                description=server.description,
                                mcp_type="command"
                            )
                            external_items.append(marketplace_item)

                        # Get marketplace items
                        marketplace_items = hub.get_marketplace_items()
                        
                        # Combine results for display
                        result_sections = []
                        
                        if builtin_servers:
                            builtin_title = get_message_with_format("mcp_list_builtin_title")
                            builtin_list = [f"- {item.name}" for item in builtin_servers]
                            result_sections.append(builtin_title)
                            result_sections.append("\n".join(builtin_list))
                            
                        if external_items:
                            external_title = get_message_with_format("mcp_list_external_title")
                            external_list = [f"- {item.name} ({item.description})" for item in external_items]
                            result_sections.append(external_title)
                            result_sections.append("\n".join(external_list))
                            
                        if marketplace_items:
                            marketplace_title = get_message_with_format("mcp_list_marketplace_title")
                            marketplace_list = [f"- {item.name} ({item.description})" for item in marketplace_items]
                            result_sections.append(marketplace_title)
                            result_sections.append("\n".join(marketplace_list))
                        
                        result = "\n\n".join(result_sections)
                        
                        # Create raw result with MarketplaceMCPServerItem objects
                        raw_result = ListResult(
                            builtin_servers=builtin_servers,
                            external_servers=external_items,
                            marketplace_items=marketplace_items
                        )

                        await self._response_queue.put(McpResponse(result=result, raw_result=raw_result))
                    except Exception as e:
                        await self._response_queue.put(McpResponse(
                            result="", 
                            error=get_message_with_format("mcp_list_builtin_error", error=str(e)),
                            raw_result=ListResult(error=str(e))
                        ))

                elif isinstance(request, McpServerInfoRequest):
                    try:
                        llm = get_single_llm(request.model, product_mode=request.product_mode or "lite")
                        if llm is None:
                            raise ValueError("Failed to get LLM instance")
                        mcp_executor = McpExecutor(hub, llm)
                        result = mcp_executor.get_connected_servers_info()
                        await self._response_queue.put(McpResponse(
                            result=result, 
                            raw_result=StringResult(result=result)
                        ))
                    except Exception as e:
                        import traceback
                        traceback.print_exc()
                        await self._response_queue.put(McpResponse(
                            result="", 
                            error=get_message_with_format("mcp_server_info_error", error=str(e)),
                            raw_result=ErrorResult(error=str(e))
                        ))

                elif isinstance(request, McpListRunningRequest):
                    try:
                        servers = hub.get_servers()
                        running_servers = "\n".join([f"- {server.name}" for server in servers])
                        result = running_servers if running_servers else ""
                        await self._response_queue.put(McpResponse(
                            result=result,
                            raw_result=ListRunningResult(
                                servers=[ServerInfo(name=server.name) for server in servers]
                            )
                        ))
                    except Exception as e:
                        await self._response_queue.put(McpResponse(
                            result="", 
                            error=get_message_with_format("mcp_list_running_error", error=str(e)),
                            raw_result=ListRunningResult(error=str(e))
                        ))

                elif isinstance(request, McpRefreshRequest):
                    try:
                        if request.name:
                            await hub.refresh_server_connection(request.name)
                        else:
                            await hub.initialize()
                        await self._response_queue.put(McpResponse(
                            result=get_message_with_format("mcp_refresh_success"),
                            raw_result=RefreshResult(
                                success=True,
                                name=request.name
                            )
                        ))
                    except Exception as e:
                        await self._response_queue.put(McpResponse(
                            result="", 
                            error=get_message_with_format("mcp_refresh_error", error=str(e)),
                            raw_result=RefreshResult(
                                success=False,
                                name=request.name,
                                error=str(e)
                            )
                        ))
                
                elif isinstance(request, MarketplaceAddRequest):
                    try:
                        # Create a MarketplaceMCPServerItem from the request
                        item = MarketplaceMCPServerItem(
                            name=request.name,
                            description=request.description,
                            mcp_type=request.mcp_type,
                            command=request.command or "",
                            args=request.args or [],
                            env=request.env or {},
                            url=request.url or ""
                        )
                        
                        # Add the item to the marketplace
                        success = await hub.add_marketplace_item(item)
                        
                        if success:
                            await self._response_queue.put(McpResponse(
                                result=get_message_with_format("marketplace_add_success", name=request.name),
                                raw_result=MarketplaceAddResult(
                                    success=True,
                                    name=request.name
                                )
                            ))
                        else:
                            await self._response_queue.put(McpResponse(
                                result="", 
                                error=get_message_with_format("marketplace_add_error", name=request.name),
                                raw_result=MarketplaceAddResult(
                                    success=False,
                                    name=request.name,
                                    error=f"Failed to add marketplace item: {request.name}"
                                )
                            ))
                    except Exception as e:
                        await self._response_queue.put(McpResponse(
                            result="", 
                            error=get_message_with_format("marketplace_add_error", name=request.name, error=str(e)),
                            raw_result=MarketplaceAddResult(
                                success=False,
                                name=request.name,
                                error=str(e)
                            )
                        ))

                elif isinstance(request, MarketplaceUpdateRequest):
                    try:
                        # Create a MarketplaceMCPServerItem from the request
                        item = MarketplaceMCPServerItem(
                            name=request.name,
                            description=request.description,
                            mcp_type=request.mcp_type,
                            command=request.command or "",
                            args=request.args or [],
                            env=request.env or {},
                            url=request.url or ""
                        )
                        
                        # Update the item in the marketplace
                        success = await hub.update_marketplace_item(request.name, item)
                        
                        if success:
                            await self._response_queue.put(McpResponse(
                                result=get_message_with_format("marketplace_update_success", name=request.name),
                                raw_result=MarketplaceUpdateResult(
                                    success=True,
                                    name=request.name
                                )
                            ))
                        else:
                            await self._response_queue.put(McpResponse(
                                result="", 
                                error=get_message_with_format("marketplace_update_error", name=request.name),
                                raw_result=MarketplaceUpdateResult(
                                    success=False,
                                    name=request.name,
                                    error=f"Failed to update marketplace item: {request.name}"
                                )
                            ))
                    except Exception as e:
                        await self._response_queue.put(McpResponse(
                            result="", 
                            error=get_message_with_format("marketplace_update_error", name=request.name, error=str(e)),
                            raw_result=MarketplaceUpdateResult(
                                success=False,
                                name=request.name,
                                error=str(e)
                            )
                        ))

                else:
                    # Handle general MCP requests
                    if not request.query.strip():
                        await self._response_queue.put(McpResponse(
                            result="", 
                            error=get_message_with_format("mcp_query_empty"),
                            raw_result=QueryResult(
                                success=False,
                                error="Empty query"
                            )
                        ))
                        continue

                    llm = get_single_llm(request.model, product_mode=request.product_mode)
                    mcp_executor = McpExecutor(hub, llm)
                    conversations = [{"role": "user", "content": request.query}]
                    _, results = await mcp_executor.run(conversations)

                    if not results:
                        await self._response_queue.put(McpResponse(
                            result=get_message_with_format("mcp_error_title"),
                            error="No results",
                            raw_result=QueryResult(
                                success=False,
                                error="No results"
                            )
                        ))
                    else:
                        results_str = "\n\n".join(
                            mcp_executor.format_mcp_result(result) for result in results)
                        await self._response_queue.put(McpResponse(
                            result=get_message_with_format("mcp_response_title") + "\n" + results_str,
                            raw_result=QueryResult(
                                success=True,
                                results=results
                            )
                        ))
            except asyncio.CancelledError:
                logger.info("MCP request processing was cancelled")
                break
            except Exception as e:
                logger.error(f"Error in MCP request processing: {e}")
                try:
                    await self._response_queue.put(McpResponse(
                        result="", 
                        error=get_message_with_format("mcp_error_title") + ": " + str(e),
                        raw_result=ErrorResult(error=str(e))
                    ))
                except Exception:
                    pass
        
        # Clean up hub connections
        try:
            await hub.shutdown()
        except Exception as e:
            logger.error(f"Error shutting down MCP hub: {e}")

    def send_request(self, request: Union[McpRequest, McpServerInfoRequest]) -> McpResponse:
        """
        Send a request to the MCP server and wait for response
        
        Args:
            request: The request to send
            
        Returns:
            McpResponse: The response from the server
        """
        if not self._running:
            self.start()
        
        async def _send():
            await self._request_queue.put(request)
            return await self._response_queue.get()
        
        if self._loop is None:
            return McpResponse(result="", error="Event loop not available")
        
        try:
            future = asyncio.run_coroutine_threadsafe(_send(), self._loop)
            return future.result(timeout=30.0)  # Add timeout to prevent hanging
        except asyncio.TimeoutError:
            return McpResponse(result="", error="Request timeout")
        except Exception as e:
            return McpResponse(result="", error=f"Request failed: {str(e)}")


# Global MCP server instance
_mcp_server = None


def get_mcp_server():
    """Get the global MCP server instance"""
    global _mcp_server
    if _mcp_server is None:
        _mcp_server = McpServer()
        _mcp_server.start()
        # Register cleanup function
        atexit.register(_cleanup_mcp_server)
    return _mcp_server


def _cleanup_mcp_server():
    """Clean up the global MCP server instance"""
    global _mcp_server
    if _mcp_server is not None:
        logger.info("Cleaning up MCP server...")
        _mcp_server.stop()
        _mcp_server = None



from typing import List, Dict, Tuple
from autocoder.common.types import Mode, CodeGenerateResult
from autocoder.common import AutoCoderArgs
import byzerllm
from autocoder.common import sys_prompt
from autocoder.privacy.model_filter import ModelPathFilter
import json
from concurrent.futures import ThreadPoolExecutor
from autocoder.common.utils_code_auto_generate import (
    chat_with_continue,
    stream_chat_with_continue,
    ChatWithContinueResult,
)
from autocoder.utils.auto_coder_utils.chat_stream_out import stream_out
from autocoder.common.stream_out_type import CodeGenerateStreamOutType
from autocoder.common.auto_coder_lang import get_message_with_format
from autocoder.common.printer import Printer
from autocoder.common.tokens import count_string_tokens as count_tokens
from autocoder.utils import llms as llm_utils
from autocoder.common import SourceCodeList
from autocoder.memory.active_context_manager import ActiveContextManager
from loguru import logger
from autocoder.common.rulefiles import get_rules
from autocoder.run_context import get_run_context, RunMode
import os
from datetime import datetime
import platform


class CodeAutoGenerate:
    def __init__(
        self,
        llm: byzerllm.ByzerLLM,
        args: AutoCoderArgs,
        action=None,
    ) -> None:
        self.llm = llm
        self.args = args
        self.action = action
        self.generate_times_same_model = args.generate_times_same_model
        if not self.llm:
            raise ValueError(
                "Please provide a valid model instance to use for code generation."
            )
        self.llms = self.llm.get_sub_client("code_model") or [self.llm]
        if not isinstance(self.llms, list):
            self.llms = [self.llms]

    @byzerllm.prompt(llm=lambda self: self.llm)
    def single_round_instruction(
        self,
        instruction: str,
        content: str,
        context: str = "",
        package_context: str = "",
    ) -> str:
        """
        <env>
        当前工作目录: {{ current_dir }}
        操作系统: {{ os_info }}
        当前时间: {{ current_time }}
        </env>

        {%- if structure %}
        ====
        {{ structure }}
        {%- endif %}

        {%- if content %}
        ====
        下面是一些文件路径以及每个文件对应的源码：

        {{ content }}
        {%- endif %}

        {%- if package_context %}
        ====
        下面是上面文件的一些信息（包括最近的变更情况）：
        <package_context>
        {{ package_context }}
        </package_context>
        {%- endif %}

        {%- if context %}
        ====
        {{ context }}
        {%- endif %}

        {%- if extra_docs %}
        ====

        RULES PROVIDED BY USER

        The following rules are provided by the user, and you must follow them strictly.

        {% for key, value in extra_docs.items() %}
        <user_rule>
        ##File: {{ key }}
        {{ value }}
        </user_rule>
        {% endfor %}
        {% endif %}

        下面是用户的需求：

        {{ instruction }}

        如果你需要生成代码，你生成的代码要符合这个格式：

        ```{lang}
        ##File: {FILE_PATH}
        {CODE}
        ```

        ```{lang}
        ##File: {FILE_PATH}
        {CODE}
        ```

        其中，{lang}是代码的语言，{CODE}是代码的内容, {FILE_PATH} 是文件的路径(请尽量使用绝对路径)，他们都在代码块中，请严格按上面的格式进行内容生成。

        请确保每份代码的完整性，而不要只生成修改部分。
        """
        # 收集环境信息
        current_dir = os.getcwd()
        os_info = f"{platform.system()} {platform.release()}"
        current_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

        if not self.args.include_project_structure:
            return {
                "structure": "",
                "current_dir": current_dir,
                "os_info": os_info,
                "current_time": current_time,
            }

        extra_docs = get_rules()

        return {
            "structure": (
                self.action.pp.get_tree_like_directory_structure()
                if self.action
                else ""
            ),
            "extra_docs": extra_docs,
            "current_dir": current_dir,
            "os_info": os_info,
            "current_time": current_time,
        }

    def single_round_run(
        self, query: str, source_code_list: SourceCodeList
    ) -> CodeGenerateResult:

        # Apply model filter for code_llm
        printer = Printer()
        for llm in self.llms:
            model_filter = ModelPathFilter.from_model_object(llm, self.args)
            filtered_sources = []
            for source in source_code_list.sources:
                if model_filter.is_accessible(source.module_name):
                    filtered_sources.append(source)
                else:
                    printer.print_in_terminal(
                        "index_file_filtered",
                        style="yellow",
                        file_path=source.module_name,
                        model_name=",".join(llm_utils.get_llm_names(llm)),
                    )

        source_code_list = SourceCodeList(filtered_sources)

        llm_config = {"human_as_model": self.args.human_as_model}

        source_content = source_code_list.to_str()

        active_context_manager = ActiveContextManager(self.llm, self.args.source_dir)

        # 获取包上下文信息
        package_context = ""

        if (
            self.args.enable_active_context
            and self.args.enable_active_context_in_generate
        ):
            # 获取活动上下文信息
            result = active_context_manager.load_active_contexts_for_files(
                [source.module_name for source in source_code_list.sources]
            )
            # 将活动上下文信息格式化为文本
            if result.contexts:
                package_context_parts = []
                for dir_path, context in result.contexts.items():
                    package_context_parts.append(
                        f"<package_info>{context.content}</package_info>"
                    )

                package_context = "\n".join(package_context_parts)

        init_prompt = self.single_round_instruction.prompt(
            instruction=query,
            content=source_content,
            context=self.args.context,
            package_context=package_context,
        )

        with open(self.args.target_file, "w", encoding="utf-8") as file:
            file.write(init_prompt)

        conversations = []

        if self.args.system_prompt and self.args.system_prompt.strip() == "claude":
            conversations.append(
                {"role": "system", "content": sys_prompt.claude_sys_prompt.prompt()}
            )
        elif self.args.system_prompt:
            conversations.append({"role": "system", "content": self.args.system_prompt})

        conversations.append({"role": "user", "content": init_prompt})

        conversations_list = []
        results = []
        input_tokens_count = 0
        generated_tokens_count = 0

        input_tokens_cost = 0
        generated_tokens_cost = 0

        model_names = []

        printer = Printer()
        estimated_input_tokens = count_tokens(
            json.dumps(conversations, ensure_ascii=False)
        )
        printer.print_in_terminal(
            "estimated_input_tokens_in_generate",
            style="yellow",
            estimated_input_tokens_in_generate=estimated_input_tokens,
            generate_mode="diff",
        )

        if not self.args.human_as_model or get_run_context().mode == RunMode.WEB:
            with ThreadPoolExecutor(
                max_workers=len(self.llms) * self.generate_times_same_model
            ) as executor:
                futures = []
                count = 0
                for llm in self.llms:

                    model_names_list = llm_utils.get_llm_names(llm)
                    model_name = None
                    if model_names_list:
                        model_name = model_names_list[0]

                    for _ in range(self.generate_times_same_model):
                        model_names.append(model_name)
                        if count == 0:

                            def job():
                                stream_generator = stream_chat_with_continue(
                                    llm=llm,
                                    conversations=conversations,
                                    llm_config=llm_config,
                                    args=self.args,
                                )
                                full_response, last_meta = stream_out(
                                    stream_generator,
                                    model_name=model_name,
                                    title=get_message_with_format(
                                        "code_generate_title", model_name=model_name
                                    ),
                                    args=self.args,
                                    extra_meta={
                                        "stream_out_type": CodeGenerateStreamOutType.CODE_GENERATE.value
                                    },
                                )
                                return ChatWithContinueResult(
                                    content=full_response,
                                    input_tokens_count=last_meta.input_tokens_count,
                                    generated_tokens_count=last_meta.generated_tokens_count,
                                )

                            futures.append(executor.submit(job))
                        else:
                            futures.append(
                                executor.submit(
                                    chat_with_continue,
                                    llm=llm,
                                    conversations=conversations,
                                    llm_config=llm_config,
                                    args=self.args,
                                )
                            )
                        count += 1

                temp_results = [future.result() for future in futures]

                for result, model_name in zip(temp_results, model_names):
                    results.append(result.content)
                    input_tokens_count += result.input_tokens_count
                    generated_tokens_count += result.generated_tokens_count
                    model_info = llm_utils.get_model_info(
                        model_name, self.args.product_mode
                    )
                    input_cost = model_info.get("input_price", 0) if model_info else 0
                    output_cost = model_info.get("output_price", 0) if model_info else 0
                    input_tokens_cost += (
                        input_cost * result.input_tokens_count / 1000000
                    )
                    generated_tokens_cost += (
                        output_cost * result.generated_tokens_count / 1000000
                    )

            for result in results:
                conversations_list.append(
                    conversations + [{"role": "assistant", "content": result}]
                )
        else:
            for _ in range(self.args.human_model_num):
                single_result = chat_with_continue(
                    llm=self.llms[0],
                    conversations=conversations,
                    llm_config=llm_config,
                    args=self.args,
                )
                results.append(single_result.content)
                input_tokens_count += single_result.input_tokens_count
                generated_tokens_count += single_result.generated_tokens_count
                conversations_list.append(
                    conversations
                    + [{"role": "assistant", "content": single_result.content}]
                )

        statistics = {
            "input_tokens_count": input_tokens_count,
            "generated_tokens_count": generated_tokens_count,
            "input_tokens_cost": input_tokens_cost,
            "generated_tokens_cost": generated_tokens_cost,
        }

        return CodeGenerateResult(
            contents=results, conversations=conversations_list, metadata=statistics
        )



"""
Timeout configuration module for shell command execution.

This module provides flexible timeout configuration for different types of commands
and execution scenarios.
"""

import os
import re
from dataclasses import dataclass, field
from typing import Dict, Optional, Union, List

from .exceptions import TimeoutConfigError


@dataclass
class TimeoutConfig:
    """
    Configuration class for command timeout settings.
    
    This class provides comprehensive timeout configuration including:
    - Default timeout for all commands
    - Interactive command timeout
    - Process cleanup timeout
    - Grace period for graceful termination
    - Command-specific timeout overrides
    
    Attributes:
        default_timeout: Default timeout for all commands (seconds)
        interactive_timeout: Timeout for interactive commands (seconds)
        cleanup_timeout: Timeout for process cleanup operations (seconds)
        grace_period: Grace period for graceful process termination (seconds)
        command_timeouts: Dictionary mapping command patterns to specific timeouts
    """
    
    default_timeout: Optional[float] = 300.0  # 5 minutes
    interactive_timeout: Optional[float] = 600.0  # 10 minutes
    cleanup_timeout: float = 10.0  # 10 seconds
    grace_period: float = 5.0  # 5 seconds
    
    # Command-specific timeout overrides
    command_timeouts: Dict[str, float] = field(default_factory=dict)
    
    def __post_init__(self):
        """Validate configuration after initialization."""
        self._validate_config()
    
    def _validate_config(self):
        """Validate timeout configuration values."""
        
        # Validate default_timeout
        if self.default_timeout is not None and self.default_timeout <= 0:
            raise TimeoutConfigError(
                "default_timeout", 
                self.default_timeout, 
                "must be positive or None"
            )
        
        # Validate interactive_timeout
        if self.interactive_timeout is not None and self.interactive_timeout <= 0:
            raise TimeoutConfigError(
                "interactive_timeout", 
                self.interactive_timeout, 
                "must be positive or None"
            )
        
        # Validate cleanup_timeout
        if self.cleanup_timeout <= 0:
            raise TimeoutConfigError(
                "cleanup_timeout", 
                self.cleanup_timeout, 
                "must be positive"
            )
        
        # Validate grace_period
        if self.grace_period <= 0:
            raise TimeoutConfigError(
                "grace_period", 
                self.grace_period, 
                "must be positive"
            )
        
        # Validate command_timeouts
        for command, timeout in self.command_timeouts.items():
            if timeout <= 0:
                raise TimeoutConfigError(
                    f"command_timeouts[{command}]", 
                    timeout, 
                    "must be positive"
                )
    
    def get_timeout_for_command(self, command: Union[str, List[str]]) -> Optional[float]:
        """
        Get the appropriate timeout for a specific command.
        
        This method checks for command-specific timeouts first, then falls back
        to the default timeout.
        
        Args:
            command: The command string or list to get timeout for
            
        Returns:
            Timeout in seconds, or None for no timeout
        """
        # Convert command to string if needed
        if isinstance(command, list):
            if not command:
                return self.default_timeout
            command_str = " ".join(str(arg) for arg in command)
        else:
            command_str = command
            
        if not command_str:
            return self.default_timeout
        
        # Extract the base command (first word)
        base_command = command_str.strip().split()[0] if command_str.strip() else ""
        command_parts = command_str.strip().split()
        is_single_command = len(command_parts) == 1
        
        # Check for pattern matches - prioritize patterns that match from the beginning
        # Sort patterns by specificity: exact matches first for single commands, then patterns, then exact for multi-word
        def pattern_priority(item):
            pattern, _ = item
            # For single-word commands, exact matches get highest priority
            # For multi-word commands, patterns get higher priority than base command exact matches
            is_exact = pattern == base_command
            is_pattern = '*' in pattern
            # Patterns starting with text (not *) get higher priority than wildcards
            starts_with_text = not pattern.startswith('*')
            # Longer patterns get higher priority within same category
            length = len(pattern)
            
            if is_single_command:
                # Single command: exact > patterns
                return (is_exact, not is_pattern, starts_with_text, length)
            else:
                # Multi-word command: patterns > exact base command
                return (not is_exact or is_pattern, starts_with_text, length)
        
        patterns = sorted(self.command_timeouts.items(), key=pattern_priority, reverse=True)
        
        for pattern, timeout in patterns:
            # For exact matches, check base command
            if pattern == base_command and ('*' not in pattern):
                # For multi-word commands, only return exact match if no patterns match
                if is_single_command:
                    return timeout
                # For multi-word, continue to check patterns first
            # For patterns, check full command string
            if self._matches_pattern(command_str, pattern):
                return timeout
        
        # If we get here and it's a multi-word command, check for exact base command match
        if not is_single_command and base_command in self.command_timeouts:
            pattern = base_command
            if '*' not in pattern:  # Make sure it's not a pattern
                return self.command_timeouts[base_command]
        
        # Fall back to default timeout
        return self.default_timeout
    
    def _matches_pattern(self, command: str, pattern: str) -> bool:
        """
        Check if a command matches a timeout pattern.
        
        Args:
            command: The command to check
            pattern: The pattern to match against
            
        Returns:
            True if the command matches the pattern
        """
        # Support simple wildcard patterns
        if '*' in pattern:
            # Convert simple wildcard to regex
            regex_pattern = pattern.replace('*', '.*')
            return bool(re.match(regex_pattern, command))
        
        # Support exact substring matching
        return pattern in command
    
    def set_command_timeout(self, command_pattern: str, timeout: float):
        """
        Set timeout for a specific command pattern.
        
        Args:
            command_pattern: Command or pattern to set timeout for
            timeout: Timeout value in seconds
            
        Raises:
            TimeoutConfigError: If timeout value is invalid
        """
        if timeout <= 0:
            raise TimeoutConfigError(
                f"command_timeouts[{command_pattern}]", 
                timeout, 
                "must be positive"
            )
        
        self.command_timeouts[command_pattern] = timeout
    
    def remove_command_timeout(self, command_pattern: str):
        """
        Remove timeout override for a command pattern.
        
        Args:
            command_pattern: Command pattern to remove
        """
        self.command_timeouts.pop(command_pattern, None)
    
    @classmethod
    def from_env(cls) -> 'TimeoutConfig':
        """
        Create TimeoutConfig from environment variables.
        
        Environment variables:
            AUTOCODER_DEFAULT_TIMEOUT: Default timeout in seconds
            AUTOCODER_INTERACTIVE_TIMEOUT: Interactive timeout in seconds
            AUTOCODER_CLEANUP_TIMEOUT: Cleanup timeout in seconds
            AUTOCODER_GRACE_PERIOD: Grace period in seconds
            
        Returns:
            TimeoutConfig instance configured from environment
        """
        def get_optional_float(env_var: str, default: Optional[float]) -> Optional[float]:
            value = os.getenv(env_var)
            if value is None:
                return default
            if value.lower() in ('none', 'null', ''):
                return None
            try:
                return float(value)
            except ValueError:
                raise TimeoutConfigError(
                    env_var, 
                    value, 
                    "must be a valid number or 'none'"
                )
        
        def get_float(env_var: str, default: float) -> float:
            value = os.getenv(env_var)
            if value is None:
                return default
            try:
                return float(value)
            except ValueError:
                raise TimeoutConfigError(
                    env_var, 
                    value, 
                    "must be a valid number"
                )
        
        return cls(
            default_timeout=get_optional_float('AUTOCODER_DEFAULT_TIMEOUT', 300.0),
            interactive_timeout=get_optional_float('AUTOCODER_INTERACTIVE_TIMEOUT', 600.0),
            cleanup_timeout=get_float('AUTOCODER_CLEANUP_TIMEOUT', 10.0),
            grace_period=get_float('AUTOCODER_GRACE_PERIOD', 5.0),
        )
    
    def to_dict(self) -> Dict:
        """
        Convert configuration to dictionary.
        
        Returns:
            Dictionary representation of the configuration
        """
        return {
            'default_timeout': self.default_timeout,
            'interactive_timeout': self.interactive_timeout,
            'cleanup_timeout': self.cleanup_timeout,
            'grace_period': self.grace_period,
            'command_timeouts': self.command_timeouts.copy(),
        }
    
    @classmethod
    def from_dict(cls, config_dict: Dict) -> 'TimeoutConfig':
        """
        Create TimeoutConfig from dictionary.
        
        Args:
            config_dict: Dictionary containing configuration values
            
        Returns:
            TimeoutConfig instance
        """
        return cls(
            default_timeout=config_dict.get('default_timeout', 300.0),
            interactive_timeout=config_dict.get('interactive_timeout', 600.0),
            cleanup_timeout=config_dict.get('cleanup_timeout', 10.0),
            grace_period=config_dict.get('grace_period', 5.0),
            command_timeouts=config_dict.get('command_timeouts', {}),
        )
    
    def __str__(self) -> str:
        """String representation of the configuration."""
        return (
            f"TimeoutConfig("
            f"default={self.default_timeout}, "
            f"interactive={self.interactive_timeout}, "
            f"cleanup={self.cleanup_timeout}, "
            f"grace={self.grace_period}, "
            f"overrides={len(self.command_timeouts)})"
        )
    
    def __repr__(self) -> str:
        """Detailed string representation."""
        return (
            f"TimeoutConfig("
            f"default_timeout={self.default_timeout}, "
            f"interactive_timeout={self.interactive_timeout}, "
            f"cleanup_timeout={self.cleanup_timeout}, "
            f"grace_period={self.grace_period}, "
            f"command_timeouts={self.command_timeouts})"
        ) 


"""
RAG Terminal - RAG 系统的终端交互入口模块

这个模块提供了 RAG 系统的终端交互功能，代码结构清晰，职责分明。
"""

# 延迟导入以避免循环依赖
__all__ = ["run_cli"]


def __getattr__(name):
    """延迟导入以避免循环依赖"""
    if name == "run_cli":
        from autocoder.rag.terminal.bootstrap import run_cli

        return run_cli
    raise AttributeError(f"module {__name__!r} has no attribute {name!r}")



"""
Agent Events Module

A comprehensive event handling system for auto-coder agents with support for 
event emission, subscription, filtering, and processing.
"""

from .types import (
    EventType,
    EventMessage,
    EventHandler,
    EventListener,
    EventMetrics,
    EventEmitterConfig,
    EventContext
)

from .event_factory import EventFactory
from .agent_event_emitter import AgentEventEmitter
from .utils import (
    create_event_system,
    create_pre_tool_use_event,
    create_post_tool_use_event,
    create_error_occurred_event,
    create_conversation_start_event,
    create_conversation_end_event,
    create_custom_event
)

__all__ = [
    # Types
    'EventType',
    'EventMessage', 
    'EventHandler',
    'EventListener',
    'EventMetrics',
    'EventEmitterConfig',
    'EventContext',
    
    # Core classes
    'EventFactory',
    'AgentEventEmitter',
    
    # Utility functions
    'create_event_system',
    'create_pre_tool_use_event',
    'create_post_tool_use_event', 
    'create_error_occurred_event',
    'create_conversation_start_event',
    'create_conversation_end_event',
    'create_custom_event'
] 


"""远程服务管理器 - 管理远程资源的同步和下载"""

from pathlib import Path
from typing import Optional, List, Dict, Any
from loguru import logger

from autocoder.remote_service.api_client import RemoteAPIClient
from autocoder.remote_service.models import (
    ResourceType,
    RemoteResource,
    SyncResult,
    ResourceStats,
)


# 资源类型对应的本地目录名和文件扩展名
RESOURCE_TYPE_CONFIG = {
    ResourceType.AGENTS: {
        "local_dir": ".autocoderagents",
        "extension": ".md",
    },
    ResourceType.WORKFLOWS: {
        "local_dir": ".autocoderworkflow",
        "extension": ".yaml",
    },
    ResourceType.TOOLS: {
        "local_dir": ".auto-coder/.autocodertools",
        "extension": ".md",
    },
    ResourceType.COMMANDS: {
        "local_dir": ".autocodercommands",
        "extension": ".md",
    },
}


class RemoteServiceManager:
    """远程服务管理器"""

    def __init__(
        self,
        project_root: Optional[str] = None,
        base_url: Optional[str] = None,
    ):
        """
        初始化远程服务管理器

        Args:
            project_root: 项目根目录，默认为当前目录
            base_url: API 基础 URL，默认为 https://api.auto-coder.chat
        """
        self.project_root = Path(project_root) if project_root else Path.cwd()
        self.client = RemoteAPIClient(base_url=base_url)

    def get_resources_stats(self) -> ResourceStats:
        """
        获取远程资源统计

        Returns:
            ResourceStats 对象
        """
        return self.client.get_resources_stats()

    def list_remote_resources(
        self,
        resource_type: Optional[ResourceType] = None,
    ) -> List[RemoteResource]:
        """
        列出远程资源

        Args:
            resource_type: 资源类型，None 表示所有类型

        Returns:
            远程资源列表
        """
        return self.client.get_all_resources(resource_type)

    def get_local_dir(self, resource_type: ResourceType) -> Path:
        """
        获取本地资源目录路径

        Args:
            resource_type: 资源类型

        Returns:
            本地目录 Path 对象
        """
        config = RESOURCE_TYPE_CONFIG.get(resource_type)
        if not config:
            raise ValueError(f"不支持的资源类型: {resource_type}")
        return self.project_root / config["local_dir"]

    def get_local_file_path(self, resource_type: ResourceType, filename: str) -> Path:
        """
        获取本地资源文件路径

        Args:
            resource_type: 资源类型
            filename: 文件名（可能包含扩展名）

        Returns:
            本地文件 Path 对象
        """
        config = RESOURCE_TYPE_CONFIG.get(resource_type)
        if not config:
            raise ValueError(f"不支持的资源类型: {resource_type}")

        local_dir = self.get_local_dir(resource_type)
        extension = config["extension"]

        # 去除现有扩展名，统一使用配置的扩展名
        # 例如：example-workflow.md -> example-workflow.yaml
        base_name = Path(filename).stem
        filename = f"{base_name}{extension}"

        return local_dir / filename

    def sync_resource(
        self,
        resource_type: ResourceType,
        filename: str,
        force: bool = False,
    ) -> bool:
        """
        同步单个资源

        Args:
            resource_type: 资源类型
            filename: 文件名
            force: 是否强制覆盖

        Returns:
            是否成功同步
        """
        local_path = self.get_local_file_path(resource_type, filename)

        # 检查本地文件是否存在
        if local_path.exists() and not force:
            logger.debug(f"本地文件已存在，跳过: {local_path}")
            return False

        try:
            # 下载资源内容（二进制方式）
            content_bytes = self.client.download_resource(resource_type, filename)
            if not content_bytes:
                logger.warning(f"获取资源内容为空: {filename}")
                return False

            # 确保目录存在
            local_path.parent.mkdir(parents=True, exist_ok=True)

            # 写入文件（二进制方式，支持文本和二进制文件）
            local_path.write_bytes(content_bytes)
            logger.info(f"已同步: {local_path}")
            return True

        except Exception as e:
            logger.error(f"同步资源失败 {filename}: {e}")
            return False

    def sync_all_resources(
        self,
        resource_type: Optional[ResourceType] = None,
        force: bool = False,
    ) -> SyncResult:
        """
        同步所有资源

        Args:
            resource_type: 资源类型，None 表示同步 agents 和 workflows
            force: 是否强制覆盖本地文件

        Returns:
            SyncResult 同步结果
        """
        result = SyncResult(success=True)

        types_to_sync = (
            [resource_type]
            if resource_type
            else [
                ResourceType.AGENTS,
                ResourceType.WORKFLOWS,
                ResourceType.TOOLS,
                ResourceType.COMMANDS,
            ]
        )

        for res_type in types_to_sync:
            try:
                # 获取远程资源列表
                resources = self.client.get_all_resources(res_type)
                logger.info(f"找到 {len(resources)} 个 {res_type.value} 资源")

                for resource in resources:
                    filename = resource.name
                    local_path = self.get_local_file_path(res_type, filename)

                    # 检查是否需要同步
                    if local_path.exists() and not force:
                        result.add_skipped(str(local_path))
                        continue

                    # 同步资源
                    if self.sync_resource(res_type, filename, force=force):
                        result.add_synced(str(local_path))
                    else:
                        result.add_failed(str(local_path))

            except Exception as e:
                logger.error(f"同步 {res_type.value} 资源时出错: {e}")
                result.success = False
                result.error = str(e)

        return result

    def print_resources_list(
        self,
        resource_type: Optional[ResourceType] = None,
    ) -> None:
        """
        打印远程资源列表

        Args:
            resource_type: 资源类型，None 表示所有类型
        """
        try:
            # 获取统计信息
            stats = self.get_resources_stats()
            print(f"\n📊 远程资源统计:")
            print(f"   Agents: {stats.agents_count}")
            print(f"   Workflows: {stats.workflows_count}")
            print(f"   Tools: {stats.tools_count}")
            print(f"   Commands: {stats.commands_count}")
            print(f"   总计: {stats.total_count}")

            # 获取资源列表
            resources = self.list_remote_resources(resource_type)

            if not resources:
                print("\n⚠️  没有找到远程资源")
                return

            # 按类型分组显示
            agents = [r for r in resources if r.type == ResourceType.AGENTS]
            workflows = [r for r in resources if r.type == ResourceType.WORKFLOWS]
            tools = [r for r in resources if r.type == ResourceType.TOOLS]
            commands = [r for r in resources if r.type == ResourceType.COMMANDS]

            if agents and (
                resource_type is None or resource_type == ResourceType.AGENTS
            ):
                print(f"\n📝 Agents ({len(agents)}):")
                for agent in agents:
                    desc = f" - {agent.description}" if agent.description else ""
                    print(f"   • {agent.name}{desc}")

            if workflows and (
                resource_type is None or resource_type == ResourceType.WORKFLOWS
            ):
                print(f"\n🔄 Workflows ({len(workflows)}):")
                for workflow in workflows:
                    desc = f" - {workflow.description}" if workflow.description else ""
                    print(f"   • {workflow.name}{desc}")

            if tools and (resource_type is None or resource_type == ResourceType.TOOLS):
                print(f"\n🔧 Tools ({len(tools)}):")
                for tool in tools:
                    desc = f" - {tool.description}" if tool.description else ""
                    print(f"   • {tool.name}{desc}")

            if commands and (
                resource_type is None or resource_type == ResourceType.COMMANDS
            ):
                print(f"\n⚡ Commands ({len(commands)}):")
                for cmd in commands:
                    desc = f" - {cmd.description}" if cmd.description else ""
                    print(f"   • {cmd.name}{desc}")

            print()

        except Exception as e:
            print(f"\n❌ 获取远程资源列表失败: {e}")

    def print_sync_result(self, result: SyncResult) -> None:
        """
        打印同步结果

        Args:
            result: SyncResult 对象
        """
        if result.success:
            print(f"\n✅ 同步完成!")
        else:
            print(f"\n⚠️  同步完成（有错误）")

        print(f"   已同步: {result.synced_count}")
        print(f"   已跳过: {result.skipped_count}")
        print(f"   失败: {result.failed_count}")

        if result.synced_files:
            print(f"\n📥 已同步的文件:")
            for f in result.synced_files:
                print(f"   • {f}")

        if result.skipped_files and len(result.skipped_files) <= 10:
            print(f"\n⏭️  跳过的文件（本地已存在）:")
            for f in result.skipped_files:
                print(f"   • {f}")
        elif result.skipped_files:
            print(f"\n⏭️  跳过了 {len(result.skipped_files)} 个本地已存在的文件")

        if result.failed_files:
            print(f"\n❌ 失败的文件:")
            for f in result.failed_files:
                print(f"   • {f}")

        if result.error:
            print(f"\n错误信息: {result.error}")

        print()

    def close(self):
        """关闭客户端连接"""
        self.client.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


def handle_remote_command(
    command_args: str, project_root: Optional[str] = None
) -> None:
    """
    处理 /remote 命令

    Args:
        command_args: 命令参数
        project_root: 项目根目录
    """
    args = command_args.strip().split()

    if not args or args[0] in ("help", "/help", "-h", "--help"):
        _print_remote_help()
        return

    subcommand = args[0].lstrip("/")

    with RemoteServiceManager(project_root=project_root) as manager:
        if subcommand == "resources":
            # /remote /resources [agents|workflows|tools|commands]
            resource_type = _parse_resource_type(args[1] if len(args) > 1 else None)
            manager.print_resources_list(resource_type)

        elif subcommand == "sync":
            # /remote /sync [--force] [agents|workflows|tools|commands]
            force = "--force" in args or "-f" in args
            resource_type = None

            for arg in args[1:]:
                if arg.startswith("-"):
                    continue
                resource_type = _parse_resource_type(arg)
                if resource_type:
                    break

            print(f"\n🔄 开始同步远程资源...")
            if force:
                print("   (强制覆盖模式)")

            result = manager.sync_all_resources(
                resource_type=resource_type,
                force=force,
            )
            manager.print_sync_result(result)

        else:
            print(f"\n❌ 未知的子命令: {subcommand}")
            _print_remote_help()


def _parse_resource_type(type_arg: Optional[str]) -> Optional[ResourceType]:
    """解析资源类型参数"""
    if not type_arg:
        return None

    type_lower = type_arg.lower()
    if type_lower in ("agents", "agent"):
        return ResourceType.AGENTS
    elif type_lower in ("workflows", "workflow"):
        return ResourceType.WORKFLOWS
    elif type_lower in ("tools", "tool"):
        return ResourceType.TOOLS
    elif type_lower in ("commands", "command"):
        return ResourceType.COMMANDS
    return None


def _print_remote_help() -> None:
    """打印 /remote 命令帮助"""
    help_text = """
📡 /remote - 远程资源管理

用法:
  /remote /resources [type]      列出远程可用的资源
  /remote /sync [options] [type] 同步远程资源到本地

参数:
  type        资源类型: agents | workflows | tools | commands (可选，默认全部)

选项:
  --force, -f  强制覆盖本地已存在的文件

示例:
  /remote /resources              列出所有远程资源
  /remote /resources agents       只列出 agents
  /remote /resources tools        只列出 tools
  /remote /sync                   同步所有资源（跳过已存在）
  /remote /sync --force           强制同步所有资源
  /remote /sync agents            只同步 agents
  /remote /sync --force workflows 强制同步 workflows
  /remote /sync tools             只同步 tools
  /remote /sync commands          只同步 commands

说明:
  资源将同步到以下目录:
  • Agents    -> .autocoderagents/
  • Workflows -> .autocoderworkflow/
  • Tools     -> .auto-coder/.autocodertools/
  • Commands  -> .autocodercommands/
"""
    print(help_text)



"""
AutoCoder 规则文件管理模块

提供读取、解析和监控 AutoCoder 规则文件的功能。

重构后的模块结构：
- models/: 数据模型 (RuleFile, RuleRelevance, AlwaysApplyRuleSummary, ConditionalRulesIndex)
- core/: 核心功能 (AutocoderRulesManager, RuleSelector)
- utils/: 工具函数 (parser, monitor)
- api.py: 统一的 API 接口
"""

# 导入数据模型
# 导入公共 API 函数
from .api import (
    auto_select_rules,
    generate_always_apply_summary,
    generate_conditional_rules_index,
    get_parsed_rules,
    get_required_and_index_rules,
    get_rules,
    get_rules_for_conversation,
    init_rule,
    parse_rule_file,
    reset_rules_manager,
)

# 导入核心组件
from .core import (
    AutocoderRulesManager,
    RuleSelector,
)
from .models import (
    AlwaysApplyRuleSummary,
    ConditionalRulesIndex,
    InitRule,
    RuleFile,
    RuleRelevance,
)

__all__ = [
    # 数据模型
    "RuleFile",
    "RuleRelevance",
    "AlwaysApplyRuleSummary",
    "ConditionalRulesIndex",
    "InitRule",
    # 核心组件
    "AutocoderRulesManager",
    "RuleSelector",
    # 公共 API 函数
    "get_rules",
    "get_parsed_rules",
    "parse_rule_file",
    "reset_rules_manager",
    "auto_select_rules",
    "get_required_and_index_rules",
    "generate_always_apply_summary",
    "generate_conditional_rules_index",
    "init_rule",
    "get_rules_for_conversation",
]
